#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
工厂模式测试
测试 ComponentRegistry, ComponentFactory, DefaultComponentFactory, CrawlerComponentFactory
"""
import sys
import os
import unittest
from unittest.mock import Mock, patch

# 添加项目根目录到 Python 路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.factories import (
    ComponentRegistry, 
    ComponentFactory, 
    ComponentSpec, 
    CrawlerComponentFactory,
    get_component_registry
)
from crawlo.factories.base import DefaultComponentFactory


class TestComponent:
    """测试组件类"""
    def __init__(self, name="test_component", value=42):
        self.name = name
        self.value = value


class TestFactories(unittest.TestCase):
    """工厂模式测试类"""

    def setUp(self):
        """测试前准备"""
        self.registry = ComponentRegistry()
        
    def test_component_spec_creation(self):
        """测试组件规范创建"""
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="test_component",
            component_type=TestComponent,
            factory_func=factory_func,
            dependencies=[],
            singleton=False
        )
        
        self.assertEqual(spec.name, "test_component")
        self.assertEqual(spec.component_type, TestComponent)
        self.assertEqual(spec.dependencies, [])
        self.assertFalse(spec.singleton)
        
    def test_default_component_factory(self):
        """测试默认组件工厂"""
        factory = DefaultComponentFactory()
        
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="test_component",
            component_type=TestComponent,
            factory_func=factory_func
        )
        
        # 测试创建组件
        component = factory.create(spec, name="created_component", value=100)
        self.assertIsInstance(component, TestComponent)
        self.assertEqual(component.name, "created_component")
        self.assertEqual(component.value, 100)
        
        # 测试单例模式
        spec_singleton = ComponentSpec(
            name="singleton_component",
            component_type=TestComponent,
            factory_func=factory_func,
            singleton=True
        )
        
        component1 = factory.create(spec_singleton, name="singleton_1", value=200)
        component2 = factory.create(spec_singleton, name="singleton_2", value=300)
        
        # 单例应该返回相同的实例
        self.assertIs(component1, component2)
        self.assertEqual(component1.value, 200)  # 应该是第一次创建时的值
        
    def test_component_registry_registration(self):
        """测试组件注册表注册功能"""
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="registered_component",
            component_type=TestComponent,
            factory_func=factory_func
        )
        
        # 注册组件规范
        self.registry.register(spec)
        
        # 验证注册
        retrieved_spec = self.registry.get_spec("registered_component")
        self.assertEqual(retrieved_spec, spec)
        
        # 测试列出组件
        components = self.registry.list_components()
        self.assertIn("registered_component", components)
        
    def test_component_registry_creation(self):
        """测试组件注册表创建功能"""
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="creatable_component",
            component_type=TestComponent,
            factory_func=factory_func
        )
        
        # 注册组件规范
        self.registry.register(spec)
        
        # 创建组件时应该出现错误，因为没有传递name参数
        with self.assertRaises(TypeError):
            component = self.registry.create("creatable_component", name="created", value=500)
        
    def test_global_component_registry(self):
        """测试全局组件注册表"""
        registry = get_component_registry()
        self.assertIsInstance(registry, ComponentRegistry)
        
        # 测试注册表是否包含预注册的组件
        components = registry.list_components()
        # 应该至少包含crawler组件
        self.assertGreater(len(components), 0)
        
    def test_crawler_component_factory_supports(self):
        """测试Crawler组件工厂支持检查"""
        factory = CrawlerComponentFactory()
        
        # 测试支持检查（CrawlerComponentFactory只支持特定类型）
        class Engine:
            pass
            
        class MockEngine:
            pass
            
        # Engine应该被支持
        self.assertTrue(factory.supports(Engine))
        
        # MockEngine不应该被支持
        self.assertFalse(factory.supports(MockEngine))
        
    def test_crawler_component_factory_create_without_crawler(self):
        """测试Crawler组件工厂创建时缺少crawler依赖"""
        factory = CrawlerComponentFactory()
        
        # 测试创建功能（需要crawler依赖）
        def mock_engine_factory(crawler=None, **kwargs):
            if crawler is None:
                raise ValueError("需要crawler实例")
            return "mock_engine"
            
        spec = ComponentSpec(
            name="mock_engine",
            component_type=type('Engine', (), {}),  # 使用Engine类型
            factory_func=mock_engine_factory,
            dependencies=['crawler']
        )
        
        # 测试缺少依赖时的错误处理
        with self.assertRaises(ValueError) as context:
            factory.create(spec)
        self.assertIn("Crawler instance required for component", str(context.exception))
            
    def test_crawler_component_factory_create_with_crawler(self):
        """测试Crawler组件工厂带crawler依赖的创建"""
        factory = CrawlerComponentFactory()
        
        def mock_engine_factory(crawler=None, **kwargs):
            if crawler is None:
                raise ValueError("需要crawler实例")
            return "mock_engine"
            
        spec = ComponentSpec(
            name="mock_engine",
            component_type=type('MockEngine', (), {}),
            factory_func=mock_engine_factory,
            dependencies=['crawler']
        )
        
        # 测试带依赖的创建
        result = factory.create(spec, crawler="mock_crawler")
        self.assertEqual(result, "mock_engine")
        
    def test_component_registry_clear(self):
        """测试组件注册表清空功能"""
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="clearable_component",
            component_type=TestComponent,
            factory_func=factory_func
        )
        
        # 注册组件
        self.registry.register(spec)
        self.assertIsNotNone(self.registry.get_spec("clearable_component"))
        
        # 清空注册表
        self.registry.clear()
        
        # 验证已清空
        self.assertIsNone(self.registry.get_spec("clearable_component"))
        self.assertEqual(len(self.registry.list_components()), 0)
        
    def test_default_component_factory_clear_singletons(self):
        """测试默认组件工厂清空单例功能"""
        factory = DefaultComponentFactory()
        
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec_singleton = ComponentSpec(
            name="clear_singleton_component",
            component_type=TestComponent,
            factory_func=factory_func,
            singleton=True
        )
        
        # 创建单例组件
        component1 = factory.create(spec_singleton, name="singleton_1", value=200)
        component2 = factory.create(spec_singleton, name="singleton_2", value=300)
        
        # 验证是单例
        self.assertIs(component1, component2)
        
        # 清空单例
        factory.clear_singletons()
        
        # 再次创建应该得到新实例
        component3 = factory.create(spec_singleton, name="singleton_3", value=400)
        self.assertIsNot(component1, component3)
        self.assertEqual(component3.value, 400)


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Redis Key 管理器
================
统一管理 Crawlo 框架中所有 Redis Key 的生成和验证
"""
from typing import Optional, Any

from crawlo.logging import get_logger
from crawlo.utils.redis_manager import validate_redis_key_naming


class RedisKeyManager:
    """Redis Key 管理器"""
    
    def __init__(self, project_name: str = "default", spider_name: Optional[str] = None):
        """
        初始化 Redis Key 管理器
        
        Args:
            project_name: 项目名称
            spider_name: 爬虫名称（可选）
        """
        self.project_name = project_name
        self.spider_name = spider_name
        self.logger = get_logger(self.__class__.__name__)
        
        # 如果提供了 spider_name，则使用更细粒度的命名空间
        if self.spider_name:
            self.namespace = f"{self.project_name}:{self.spider_name}"
        else:
            self.namespace = self.project_name
    
    def set_spider_name(self, spider_name: str) -> None:
        """
        设置爬虫名称
        
        Args:
            spider_name: 爬虫名称
        """
        self.spider_name = spider_name if spider_name else None
        # 更新命名空间
        if self.spider_name:
            self.namespace = f"{self.project_name}:{self.spider_name}"
        else:
            self.namespace = self.project_name
    
    def _generate_key(self, component: str, sub_component: str) -> str:
        """
        生成 Redis Key
        
        Args:
            component: 组件类型 (queue, filter, item)
            sub_component: 子组件类型
            
        Returns:
            str: 生成的 Redis Key
        """
        key = f"crawlo:{self.namespace}:{component}:{sub_component}"
        
        # 验证生成的 key 是否符合规范
        if not validate_redis_key_naming(key, self.project_name):
            self.logger.warning(f"生成的 Redis Key 不符合命名规范: {key}")
        
        return key
    
    # ==================== 队列相关 Key ====================
    
    def get_requests_queue_key(self) -> str:
        """获取请求队列 Key"""
        return self._generate_key("queue", "requests")
    
    def get_processing_queue_key(self) -> str:
        """获取处理中队列 Key"""
        return self._generate_key("queue", "processing")
    
    def get_failed_queue_key(self) -> str:
        """获取失败队列 Key"""
        return self._generate_key("queue", "failed")
    
    def get_requests_data_key(self) -> str:
        """获取请求数据 Hash Key"""
        return f"{self.get_requests_queue_key()}:data"
    
    def get_processing_data_key(self) -> str:
        """获取处理中数据 Hash Key"""
        return f"{self.get_processing_queue_key()}:data"
    
    def get_failed_retries_key(self, request_key: str) -> str:
        """获取失败重试计数 Key"""
        return f"{self.get_failed_queue_key()}:retries:{request_key}"
    
    # ==================== 过滤器相关 Key ====================
    
    def get_filter_fingerprint_key(self) -> str:
        """获取请求去重过滤器指纹 Key"""
        return self._generate_key("filter", "fingerprint")
    
    # ==================== 数据项相关 Key ====================
    
    def get_item_fingerprint_key(self) -> str:
        """获取数据项去重指纹 Key"""
        return self._generate_key("item", "fingerprint")
    
    # ==================== 静态方法 ====================
    
    @staticmethod
    def from_settings(settings: Any) -> 'RedisKeyManager':
        """
        从配置创建 Redis Key 管理器实例
        
        Args:
            settings: 配置对象
            
        Returns:
            RedisKeyManager: Redis Key 管理器实例
        """
        # 安全获取配置值
        project_name = getattr(settings, 'get', lambda k, d: getattr(settings, k, d))(
            'PROJECT_NAME', 'default'
        )
        
        # 尝试获取 spider_name，这通常在运行时才能确定
        spider_name = getattr(settings, 'get', lambda k, d: getattr(settings, k, d))(
            'SPIDER_NAME', None
        )
        
        return RedisKeyManager(project_name, spider_name)
    
    # ==================== Key 信息提取 ====================
    
    @staticmethod
    def extract_project_name_from_key(key: str) -> Optional[str]:
        """
        从 Redis Key 中提取项目名称
        
        Args:
            key: Redis Key
            
        Returns:
            Optional[str]: 项目名称
        """
        if not key or not key.startswith('crawlo:'):
            return None
            
        parts = key.split(':')
        if len(parts) >= 2:
            return parts[1]
        return None
    
    @staticmethod
    def extract_spider_name_from_key(key: str) -> Optional[str]:
        """
        从 Redis Key 中提取爬虫名称
        
        Args:
            key: Redis Key
            
        Returns:
            Optional[str]: 爬虫名称
        """
        if not key or not key.startswith('crawlo:'):
            return None
            
        parts = key.split(':')
        # 如果格式是 crawlo:project:spider:component:sub_component
        if len(parts) >= 4:
            # 检查是否存在 spider 部分
            if parts[2] not in ['queue', 'filter', 'item']:
                return parts[2]
        return None


# 便利函数
def create_redis_key_manager(project_name: str = "default", spider_name: Optional[str] = None) -> RedisKeyManager:
    """
    创建 Redis Key 管理器实例（便利函数）
    
    Args:
        project_name: 项目名称
        spider_name: 爬虫名称（可选）
        
    Returns:
        RedisKeyManager: Redis Key 管理器实例
    """
    return RedisKeyManager(project_name, spider_name)


def get_redis_key_manager_from_settings(settings: Any) -> RedisKeyManager:
    """
    从配置创建 Redis Key 管理器实例（便利函数）
    
    Args:
        settings: 配置对象
        
    Returns:
        RedisKeyManager: Redis Key 管理器实例
    """
    return RedisKeyManager.from_settings(settings)


#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Redis Key 管理器测试
==================
测试 RedisKeyManager 类的功能
"""
import unittest
from unittest.mock import MagicMock
from crawlo.utils.redis_manager import RedisKeyManager


class TestRedisKeyManager(unittest.TestCase):
    """Redis Key 管理器测试类"""
    
    def setUp(self):
        """测试初始化"""
        self.project_name = "test_project"
        self.spider_name = "test_spider"
        self.key_manager = RedisKeyManager(self.project_name, self.spider_name)
    
    def test_init_with_project_and_spider(self):
        """测试使用项目名和爬虫名初始化"""
        self.assertEqual(self.key_manager.project_name, self.project_name)
        self.assertEqual(self.key_manager.spider_name, self.spider_name)
        self.assertEqual(self.key_manager.namespace, f"{self.project_name}:{self.spider_name}")
    
    def test_init_with_project_only(self):
        """测试仅使用项目名初始化"""
        key_manager = RedisKeyManager(self.project_name)
        self.assertEqual(key_manager.project_name, self.project_name)
        self.assertIsNone(key_manager.spider_name)
        self.assertEqual(key_manager.namespace, self.project_name)
    
    def test_set_spider_name(self):
        """测试设置爬虫名称"""
        key_manager = RedisKeyManager(self.project_name)
        self.assertIsNone(key_manager.spider_name)
        self.assertEqual(key_manager.namespace, self.project_name)
        
        # 设置爬虫名称
        key_manager.set_spider_name(self.spider_name)
        self.assertEqual(key_manager.spider_name, self.spider_name)
        self.assertEqual(key_manager.namespace, f"{self.project_name}:{self.spider_name}")
        
        # 清除爬虫名称
        key_manager.set_spider_name("")
        self.assertIsNone(key_manager.spider_name)
        self.assertEqual(key_manager.namespace, self.project_name)
    
    def test_queue_keys(self):
        """测试队列相关 Key 生成"""
        # 测试请求队列 Key
        requests_queue_key = self.key_manager.get_requests_queue_key()
        expected = f"crawlo:{self.project_name}:{self.spider_name}:queue:requests"
        self.assertEqual(requests_queue_key, expected)
        
        # 测试处理中队列 Key
        processing_queue_key = self.key_manager.get_processing_queue_key()
        expected = f"crawlo:{self.project_name}:{self.spider_name}:queue:processing"
        self.assertEqual(processing_queue_key, expected)
        
        # 测试失败队列 Key
        failed_queue_key = self.key_manager.get_failed_queue_key()
        expected = f"crawlo:{self.project_name}:{self.spider_name}:queue:failed"
        self.assertEqual(failed_queue_key, expected)
        
        # 测试请求数据 Hash Key
        requests_data_key = self.key_manager.get_requests_data_key()
        expected = f"crawlo:{self.project_name}:{self.spider_name}:queue:requests:data"
        self.assertEqual(requests_data_key, expected)
        
        # 测试处理中数据 Hash Key
        processing_data_key = self.key_manager.get_processing_data_key()
        expected = f"crawlo:{self.project_name}:{self.spider_name}:queue:processing:data"
        self.assertEqual(processing_data_key, expected)
    
    def test_filter_keys(self):
        """测试过滤器相关 Key 生成"""
        # 测试过滤器指纹 Key
        filter_key = self.key_manager.get_filter_fingerprint_key()
        expected = f"crawlo:{self.project_name}:{self.spider_name}:filter:fingerprint"
        self.assertEqual(filter_key, expected)
    
    def test_item_keys(self):
        """测试数据项相关 Key 生成"""
        # 测试数据项指纹 Key
        item_key = self.key_manager.get_item_fingerprint_key()
        expected = f"crawlo:{self.project_name}:{self.spider_name}:item:fingerprint"
        self.assertEqual(item_key, expected)
    
    def test_from_settings(self):
        """测试从配置创建实例"""
        # 创建模拟配置对象
        settings = MagicMock()
        settings.get.side_effect = lambda key, default: {
            'PROJECT_NAME': self.project_name,
            'SPIDER_NAME': None
        }.get(key, default)
        
        key_manager = RedisKeyManager.from_settings(settings)
        self.assertEqual(key_manager.project_name, self.project_name)
        self.assertIsNone(key_manager.spider_name)
    
    def test_key_extraction(self):
        """测试 Key 信息提取"""
        # 测试提取项目名称
        key = f"crawlo:{self.project_name}:{self.spider_name}:queue:requests"
        extracted_project = RedisKeyManager.extract_project_name_from_key(key)
        self.assertEqual(extracted_project, self.project_name)
        
        # 测试提取爬虫名称
        extracted_spider = RedisKeyManager.extract_spider_name_from_key(key)
        self.assertEqual(extracted_spider, self.spider_name)
        
        # 测试无效 Key
        invalid_key = "invalid:key"
        extracted_project = RedisKeyManager.extract_project_name_from_key(invalid_key)
        self.assertIsNone(extracted_project)
        
        extracted_spider = RedisKeyManager.extract_spider_name_from_key(invalid_key)
        self.assertIsNone(extracted_spider)


if __name__ == '__main__':
    unittest.main()


from typing import Any
from crawlo.exceptions import NotConfigured
from crawlo.logging import get_logger

# 获取logger实例
_logger = get_logger(__name__)


class CustomLoggerExtension:
    """
    日志系统初始化扩展
    遵循与 ExtensionManager 一致的接口规范：使用 create_instance
    """

    def __init__(self, settings: Any):
        self.settings = settings
        # 使用新的日志系统，但要简化配置传递
        try:
            from crawlo.logging import configure_logging
            # 直接传递settings对象，让日志系统内部处理
            configure_logging(settings)
        except Exception as e:
            # 如果日志系统配置失败，不应该阻止扩展加载
            # 使用基本日志输出错误信息
            import logging
            logging.getLogger(__name__).warning(f"Failed to configure logging system: {e}")
            # 不抛出异常，让扩展继续加载

    @classmethod
    def create_instance(cls, crawler: Any, *args: Any, **kwargs: Any) -> 'CustomLoggerExtension':
        """
        工厂方法：兼容 ExtensionManager 的创建方式
        被 ExtensionManager 调用
        """
        # 可以通过 settings 控制是否启用
        log_file = crawler.settings.get('LOG_FILE')
        log_enable_custom = crawler.settings.get('LOG_ENABLE_CUSTOM', False)
        
        # 只有当没有配置日志文件且未启用自定义日志时才禁用
        if not log_file and not log_enable_custom:
            raise NotConfigured("CustomLoggerExtension: LOG_FILE not set and LOG_ENABLE_CUSTOM=False")

        return cls(crawler.settings)

    def spider_opened(self, spider: Any) -> None:
        try:
            _logger.info(
                f"CustomLoggerExtension: Logging initialized. "
                f"LOG_FILE={self.settings.get('LOG_FILE')}, "
                f"LOG_LEVEL={self.settings.get('LOG_LEVEL')}"
            )
        except Exception as e:
            # 即使日志初始化信息无法打印，也不应该影响程序运行
            pass


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
ResponseCodeMiddleware 测试脚本
验证ResponseCodeMiddleware的功能是否正常工作
"""

import asyncio
import sys
import os

# 添加项目根目录到Python路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.middleware.response_code import ResponseCodeMiddleware
from crawlo.network.request import Request
from crawlo.network.response import Response
from crawlo.settings.setting_manager import SettingManager


class MockStats:
    """模拟统计对象"""
    def __init__(self):
        self.counts = {}
        
    def inc_value(self, key, count=1):
        if key in self.counts:
            self.counts[key] += count
        else:
            self.counts[key] = count

    def get_value(self, key, default=None):
        return self.counts.get(key, default)


class MockLogger:
    """模拟日志对象"""
    def __init__(self):
        self.messages = []
        
    def debug(self, message):
        self.messages.append(f"DEBUG: {message}")
        
    def isEnabledFor(self, level):
        # 总是返回True以确保日志消息被记录
        return True


class MockCrawler:
    """模拟爬虫实例"""
    def __init__(self, settings):
        self.settings = settings
        self.stats = MockStats()
        self.logger = MockLogger()


def create_test_settings():
    """创建测试设置"""
    settings = SettingManager()
    settings.set("LOG_LEVEL", "DEBUG")
    return settings


# 重写ResponseCodeMiddleware的create_instance方法以使用MockLogger
class TestResponseCodeMiddleware(ResponseCodeMiddleware):
    @classmethod
    def create_instance(cls, crawler):
        """
        创建中间件实例（测试版本）
        
        Args:
            crawler: 爬虫实例
            
        Returns:
            TestResponseCodeMiddleware: 中间件实例
        """
        # 创建实例但不调用get_logger
        o = cls(stats=crawler.stats, log_level=crawler.settings.get('LOG_LEVEL'))
        # 使用MockLogger替换真实Logger
        o.logger = crawler.logger
        return o


async def test_response_code_middleware():
    """测试ResponseCodeMiddleware功能"""
    print("开始测试ResponseCodeMiddleware...")
    
    # 创建设置和中间件
    settings = create_test_settings()
    crawler = MockCrawler(settings)
    middleware = TestResponseCodeMiddleware.create_instance(crawler)
    
    print("✓ ResponseCodeMiddleware创建成功")
    
    # 创建测试请求和响应
    request = Request(url="https://example.com")
    
    # 测试不同状态码的处理
    test_cases = [
        (200, "正常响应"),
        (404, "页面未找到"),
        (500, "服务器错误"),
        (301, "重定向"),
        (403, "禁止访问"),
    ]
    
    passed = 0
    total = len(test_cases)
    
    for status_code, description in test_cases:
        try:
            response = Response(
                url="https://example.com",
                status_code=status_code,
                body=b"Test response body"
            )
            
            # 处理响应
            result = middleware.process_response(request, response, None)
            
            # 验证结果
            if result == response:
                # 检查统计信息是否正确更新
                expected_key = f'response_status_code/count/{status_code}'
                category = middleware._get_status_category(status_code)
                category_key = f'response_status_code/category/{category}'
                
                if (expected_key in crawler.stats.counts and 
                    crawler.stats.counts[expected_key] == 1 and
                    category_key in crawler.stats.counts):
                    print(f"✓ {description}: 状态码 {status_code} (统计信息正确)")
                    passed += 1
                else:
                    print(f"✗ {description}: 状态码 {status_code} (统计信息错误)")
            else:
                print(f"✗ {description}: 状态码 {status_code} (返回结果不正确)")
                
        except Exception as e:
            print(f"✗ {description}: 状态码 {status_code} (出现异常: {e})")
    
    # 输出测试结果
    print(f"\n测试结果: {passed}/{total} 通过")
    if passed == total:
        print("✓ 所有测试通过，ResponseCodeMiddleware工作正常")
        return True
    else:
        print("✗ 部分测试失败，ResponseCodeMiddleware存在问题")
        return False


def test_middleware_creation():
    """测试中间件创建功能"""
    print("\n测试中间件创建功能...")
    
    try:
        settings = create_test_settings()
        crawler = MockCrawler(settings)
        middleware = TestResponseCodeMiddleware.create_instance(crawler)
        
        if middleware and hasattr(middleware, 'stats') and hasattr(middleware, 'logger'):
            print("✓ 中间件创建成功，属性完整")
            return True
        else:
            print("✗ 中间件创建失败，缺少必要属性")
            return False
    except Exception as e:
        print(f"✗ 中间件创建时出现异常: {e}")
        return False


async def test_status_category_function():
    """测试状态码分类功能"""
    print("\n测试状态码分类功能...")
    
    try:
        settings = create_test_settings()
        crawler = MockCrawler(settings)
        middleware = TestResponseCodeMiddleware.create_instance(crawler)
        
        # 测试分类功能
        test_cases = [
            (200, "2xx"),
            (299, "2xx"),
            (301, "3xx"),
            (404, "4xx"),
            (500, "5xx"),
            (100, "other"),
            (600, "other"),
        ]
        
        passed = 0
        total = len(test_cases)
        
        for status_code, expected_category in test_cases:
            category = middleware._get_status_category(status_code)
            if category == expected_category:
                print(f"✓ 状态码 {status_code} 分类为 {category}")
                passed += 1
            else:
                print(f"✗ 状态码 {status_code} 分类错误: 期望 {expected_category}, 实际 {category}")
        
        print(f"\n分类测试结果: {passed}/{total} 通过")
        if passed == total:
            print("✓ 状态码分类功能正常")
            return True
        else:
            print("✗ 状态码分类功能存在问题")
            return False
            
    except Exception as e:
        print(f"✗ 状态码分类测试时出现异常: {e}")
        return False


async def test_response_type_check_functions():
    """测试响应类型检查功能"""
    print("\n测试响应类型检查功能...")
    
    try:
        settings = create_test_settings()
        crawler = MockCrawler(settings)
        middleware = TestResponseCodeMiddleware.create_instance(crawler)
        
        # 测试成功响应检查
        success_codes = [200, 201, 299]
        for code in success_codes:
            if middleware._is_success_response(code):
                print(f"✓ 状态码 {code} 正确识别为成功响应")
            else:
                print(f"✗ 状态码 {code} 未能正确识别为成功响应")
        
        # 测试重定向响应检查
        redirect_codes = [301, 302, 307]
        for code in redirect_codes:
            if middleware._is_redirect_response(code):
                print(f"✓ 状态码 {code} 正确识别为重定向响应")
            else:
                print(f"✗ 状态码 {code} 未能正确识别为重定向响应")
        
        # 测试客户端错误检查
        client_error_codes = [400, 404, 499]
        for code in client_error_codes:
            if middleware._is_client_error(code):
                print(f"✓ 状态码 {code} 正确识别为客户端错误")
            else:
                print(f"✗ 状态码 {code} 未能正确识别为客户端错误")
        
        # 测试服务器错误检查
        server_error_codes = [500, 502, 599]
        for code in server_error_codes:
            if middleware._is_server_error(code):
                print(f"✓ 状态码 {code} 正确识别为服务器错误")
            else:
                print(f"✗ 状态码 {code} 未能正确识别为服务器错误")
        
        print("✓ 响应类型检查功能正常")
        return True
        
    except Exception as e:
        print(f"✗ 响应类型检查测试时出现异常: {e}")
        return False


async def test_logging_functionality():
    """测试日志功能"""
    print("\n测试日志功能...")
    
    try:
        settings = create_test_settings()
        crawler = MockCrawler(settings)
        middleware = TestResponseCodeMiddleware.create_instance(crawler)
        
        # 检查logger对象
        print(f"  Logger对象类型: {type(middleware.logger)}")
        print(f"  Logger是否有debug方法: {hasattr(middleware.logger, 'debug')}")
        print(f"  Logger是否有isEnabledFor方法: {hasattr(middleware.logger, 'isEnabledFor')}")
        
        # 创建测试请求和响应
        request = Request(url="https://example.com")
        response = Response(
            url="https://example.com",
            status_code=200,
            body=b"Test response body"
        )
        
        # 检查调用前的消息数量
        initial_message_count = len(crawler.logger.messages)
        print(f"  调用前消息数量: {initial_message_count}")
        
        # 处理响应
        middleware.process_response(request, response, None)
        
        # 检查调用后的消息数量
        final_message_count = len(crawler.logger.messages)
        print(f"  调用后消息数量: {final_message_count}")
        
        # 检查是否有日志消息
        if final_message_count > initial_message_count:
            print("✓ 日志功能正常工作")
            # 打印一条日志消息作为示例
            print(f"  示例日志: {crawler.logger.messages[-1]}")
            return True
        else:
            print("✗ 未生成日志消息")
            print(f"  所有消息: {crawler.logger.messages}")
            return False
    except Exception as e:
        print(f"✗ 日志测试时出现异常: {e}")
        import traceback
        traceback.print_exc()
        return False


async def main():
    """主测试函数"""
    print("ResponseCodeMiddleware 功能测试")
    print("=" * 50)
    
    # 测试正常功能
    test1_result = await test_response_code_middleware()
    
    # 测试创建功能
    test2_result = test_middleware_creation()
    
    # 测试状态码分类功能
    test3_result = await test_status_category_function()
    
    # 测试响应类型检查功能
    test4_result = await test_response_type_check_functions()
    
    # 测试日志功能
    test5_result = await test_logging_functionality()
    
    # 汇总结果
    print("\n" + "=" * 50)
    print("测试结果汇总:")
    print(f"基本功能测试: {'✓ 通过' if test1_result else '✗ 失败'}")
    print(f"创建测试: {'✓ 通过' if test2_result else '✗ 失败'}")
    print(f"分类功能测试: {'✓ 通过' if test3_result else '✗ 失败'}")
    print(f"类型检查测试: {'✓ 通过' if test4_result else '✗ 失败'}")
    print(f"日志测试: {'✓ 通过' if test5_result else '✗ 失败'}")
    
    overall_result = test1_result and test2_result and test3_result and test4_result and test5_result
    print(f"\n总体结果: {'✓ 所有测试通过' if overall_result else '✗ 部分测试失败'}")
    
    return overall_result


if __name__ == "__main__":
    asyncio.run(main())


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
# tests/test_proxy_stats.py
from crawlo.proxy.stats import ProxyStats


def test_proxy_stats():
    """测试代理统计功能"""
    stats = ProxyStats()
    url = 'http://proxy1:8080'

    stats.record(url, 'success')
    stats.record(url, 'success')
    stats.record(url, 'failure')

    assert stats.get(url)['success'] == 2
    assert stats.get(url)['failure'] == 1
    assert stats.get(url)['total'] == 3

    all_data = stats.all()
    assert url in all_data
    assert all_data[url]['success'] == 2


#!/usr/bin/python
# -*- coding:UTF-8 -*-
import traceback
from typing import Optional, Callable

from crawlo.logging import get_logger
from crawlo.project import common_call
from crawlo.utils.misc import load_object
from crawlo.utils.request import set_request
from crawlo.utils.error_handler import ErrorHandler
from crawlo.utils.request_serializer import RequestSerializer
from crawlo.queue.queue_manager import QueueManager, QueueConfig, QueueType


class Scheduler:
    def __init__(self, crawler, dupe_filter, stats, priority):
        self.crawler = crawler
        self.queue_manager: Optional[QueueManager] = None
        self.request_serializer = RequestSerializer()

        self.logger = get_logger(self.__class__.__name__)
        self.error_handler = ErrorHandler(self.__class__.__name__)
        self.stats = stats
        self.dupe_filter = dupe_filter
        self.priority = priority

    @classmethod
    def create_instance(cls, crawler):
        # 使用工具模块中的安全获取配置函数
        from crawlo.utils.misc import safe_get_config
        
        # 安全获取FILTER_CLASS设置 - 简化版本
        filter_class = safe_get_config(
            getattr(crawler, 'settings', None), 
            'FILTER_CLASS', 
            'crawlo.filters.memory_filter.MemoryFilter'
        )
        
        # 安全获取DEPTH_PRIORITY设置
        priority = safe_get_config(
            getattr(crawler, 'settings', None), 
            'DEPTH_PRIORITY', 
            0
        )
            
        filter_cls = load_object(filter_class)
        
        o = cls(
            crawler=crawler,
            dupe_filter=filter_cls.create_instance(crawler),
            stats=getattr(crawler, 'stats', None),
            priority=priority
        )
        return o

    async def open(self):
        """Initialize scheduler and queue"""
        self.logger.debug("开始初始化调度器...")
        try:
            # 如果是Redis队列，设置spider_name
            if self.crawler.spider:
                spider_name = getattr(self.crawler.spider, 'name', None)
                if spider_name:
                    # 设置SPIDER_NAME到配置中，以便RedisKeyManager能够获取
                    if hasattr(self.crawler.settings, 'set'):
                        try:
                            self.crawler.settings.set('SPIDER_NAME', spider_name)
                        except Exception:
                            pass
            
            # 创建队列配置
            queue_config = QueueConfig.from_settings(self.crawler.settings)
            
            # 创建队列管理器
            self.queue_manager = QueueManager(queue_config)
            
            # 初始化队列
            needs_config_update = await self.queue_manager.initialize()
            
            # 初始化默认配置值
            queue_type_setting = 'memory'  # 默认值
            current_filter = ''  # 默认值
            concurrency = 8  # 默认值
            delay = 1.0  # 默认值
            
            # 检查是否需要更新过滤器配置
            updated_configs = []
            if needs_config_update:
                # 如果返回True，说明队列类型发生了变化，需要检查当前队列类型来决定更新方向
                if self.queue_manager._queue_type == QueueType.REDIS:
                    self._switch_to_redis_config()
                    updated_configs.append("Redis")
                else:
                    self._switch_to_memory_config()
                    updated_configs.append("内存")
            else:
                # 检查是否需要更新配置（即使队列管理器没有要求更新）
                # 当 QUEUE_TYPE 明确设置为 redis 时，也应该检查配置一致性
                
                # 安全获取配置
                if self.crawler and self.crawler.settings is not None:
                    try:
                        queue_type_setting = self.crawler.settings.get('QUEUE_TYPE', 'memory')
                        current_filter = self.crawler.settings.get('FILTER_CLASS', '')
                        concurrency = self.crawler.settings.get('CONCURRENCY', 8)
                        delay = self.crawler.settings.get('DOWNLOAD_DELAY', 1.0)
                    except Exception:
                        # 使用默认值
                        pass
                
                if queue_type_setting == 'redis' or needs_config_update:
                    updated_configs = self._check_filter_config()
                else:
                    updated_configs = []
            
            # 处理过滤器配置更新
            await self._process_filter_updates(needs_config_update, updated_configs)
            
            # 输出关键的调度器初始化完成信息
            status = self.queue_manager.get_status() if self.queue_manager else {'type': 'unknown', 'health': 'unknown'}
            
            # 获取更新后的过滤器配置
            updated_filter = current_filter
            if self.crawler and self.crawler.settings is not None:
                try:
                    updated_filter = self.crawler.settings.get('FILTER_CLASS', current_filter)
                except Exception:
                    pass
            self.logger.info(f"enabled filters: \n  {updated_filter}")
            
            # 优化日志输出，将多条日志合并为1条关键信息
            if queue_type_setting in ['auto', 'redis'] and updated_configs:
                self.logger.debug(f"Scheduler initialized [Queue type: {status['type']}, Status: {status['health']}, Concurrency: {concurrency}, Delay: {delay}s]")
            else:
                self.logger.debug(f"Scheduler initialized [Queue type: {status['type']}, Status: {status['health']}]")
        except Exception as e:
            self.logger.error(f"Scheduler initialization failed: {e}")
            self.logger.debug(f"Detailed error information:\n{traceback.format_exc()}")
            raise
    
    def _check_filter_config(self):
        """检查并更新过滤器配置"""
        updated_configs = []
        
        # 安全检查queue_manager是否存在
        if not self.queue_manager:
            return updated_configs
            
        # 安全获取FILTER_CLASS配置
        current_filter_class = ''
        if self.crawler and self.crawler.settings is not None:
            try:
                current_filter_class = self.crawler.settings.get('FILTER_CLASS', '')
            except Exception:
                current_filter_class = ''
            
        if self.queue_manager._queue_type == QueueType.REDIS:
            # 检查当前过滤器是否为内存过滤器
            if 'memory_filter' in current_filter_class:
                self._switch_to_redis_config()
                updated_configs.append("Redis")
        elif self.queue_manager._queue_type == QueueType.MEMORY:
            # 检查当前过滤器是否为Redis过滤器
            if 'aioredis_filter' in current_filter_class or 'redis_filter' in current_filter_class:
                self._switch_to_memory_config()
                updated_configs.append("内存")
                
        return updated_configs
    
    async def _process_filter_updates(self, needs_config_update, updated_configs):
        """处理过滤器更新逻辑"""
        # 安全检查queue_manager是否存在
        if not self.queue_manager:
            return
            
        # 安全获取FILTER_CLASS配置
        current_filter_class = ''
        if self.crawler and self.crawler.settings is not None:
            try:
                current_filter_class = self.crawler.settings.get('FILTER_CLASS', '')
            except Exception:
                current_filter_class = ''
        
        filter_matches_queue_type = self._is_filter_matching_queue_type(current_filter_class)
        
        # 只有在配置不匹配且需要更新时才重新创建过滤器实例
        if needs_config_update or not filter_matches_queue_type:
            # 如果需要更新配置，则执行更新
            if needs_config_update:
                # 安全获取FILTER_CLASS配置
                filter_class = 'crawlo.filters.memory_filter.MemoryFilter'
                if self.crawler and self.crawler.settings is not None:
                    try:
                        filter_class = self.crawler.settings.get('FILTER_CLASS', filter_class)
                    except Exception:
                        pass
                
                # 重新创建过滤器实例，确保使用更新后的配置
                filter_cls = load_object(filter_class)
                self.dupe_filter = filter_cls.create_instance(self.crawler)
                
                # 记录警告信息
                original_mode = "standalone" if 'memory_filter' in current_filter_class else "distributed"
                new_mode = "distributed" if self.queue_manager._queue_type == QueueType.REDIS else "standalone"
                if original_mode != new_mode:
                    self.logger.warning(f"runtime mode inconsistency detected: switched from {original_mode} to {new_mode} mode")
            elif not filter_matches_queue_type:
                # 配置不匹配，需要更新
                if self.queue_manager._queue_type == QueueType.REDIS:
                    self._switch_to_redis_config()
                elif self.queue_manager._queue_type == QueueType.MEMORY:
                    self._switch_to_memory_config()
                
                # 安全获取FILTER_CLASS配置
                filter_class = 'crawlo.filters.memory_filter.MemoryFilter'
                if self.crawler and self.crawler.settings is not None:
                    try:
                        filter_class = self.crawler.settings.get('FILTER_CLASS', filter_class)
                    except Exception:
                        pass
                
                # 重新创建过滤器实例
                filter_cls = load_object(filter_class)
                self.dupe_filter = filter_cls.create_instance(self.crawler)
    
    def _is_filter_matching_queue_type(self, current_filter_class):
        """检查过滤器配置是否与队列类型匹配"""
        # 安全检查queue_manager是否存在
        if not self.queue_manager:
            return False
            
        return (
            (self.queue_manager._queue_type == QueueType.REDIS and 
             ('aioredis_filter' in current_filter_class or 'redis_filter' in current_filter_class)) or
            (self.queue_manager._queue_type == QueueType.MEMORY and 
             'memory_filter' in current_filter_class)
        )
    
    def _switch_to_redis_config(self):
        """切换到Redis配置"""
        # 安全检查queue_manager是否存在且类型正确
        if not self.queue_manager or self.queue_manager._queue_type != QueueType.REDIS:
            return
            
        # 安全获取FILTER_CLASS配置
        current_filter_class = ''
        default_dedup_pipeline = ''
        pipelines = []
        
        if self.crawler and self.crawler.settings is not None:
            try:
                current_filter_class = self.crawler.settings.get('FILTER_CLASS', '')
                default_dedup_pipeline = self.crawler.settings.get('DEFAULT_DEDUP_PIPELINE', '')
                pipelines = self.crawler.settings.get('PIPELINES', [])
            except Exception:
                pass
            
        updated_configs = []
        
        if 'memory_filter' in current_filter_class:
            # 更新为Redis过滤器
            if self.crawler and self.crawler.settings is not None:
                try:
                    self.crawler.settings.set('FILTER_CLASS', 'crawlo.filters.aioredis_filter.AioRedisFilter')
                    updated_configs.append("filter")
                except Exception:
                    pass
        
        # 检查当前去重管道是否为内存去重管道
        if 'memory_dedup_pipeline' in default_dedup_pipeline:
            # 更新为Redis去重管道
            if self.crawler and self.crawler.settings is not None:
                try:
                    self.crawler.settings.set('DEFAULT_DEDUP_PIPELINE', 'crawlo.pipelines.redis_dedup_pipeline.RedisDedupPipeline')
                    # 同时更新PIPELINES列表中的去重管道
                    if default_dedup_pipeline in pipelines:
                        # 找到并替换内存去重管道为Redis去重管道
                        index = pipelines.index(default_dedup_pipeline)
                        pipelines[index] = 'crawlo.pipelines.redis_dedup_pipeline.RedisDedupPipeline'
                        self.crawler.settings.set('PIPELINES', pipelines)
                    updated_configs.append("dedup pipeline")
                except Exception:
                    pass
        
        # 合并日志输出
        if updated_configs:
            self.logger.info(f"configuration updated: {', '.join(updated_configs)} -> redis mode")

    def _switch_to_memory_config(self):
        """切换到内存配置"""
        # 安全检查queue_manager是否存在且类型正确
        if not self.queue_manager or self.queue_manager._queue_type != QueueType.MEMORY:
            return
            
        # 安全获取FILTER_CLASS配置
        current_filter_class = ''
        default_dedup_pipeline = ''
        pipelines = []
        
        if self.crawler and self.crawler.settings is not None:
            try:
                current_filter_class = self.crawler.settings.get('FILTER_CLASS', '')
                default_dedup_pipeline = self.crawler.settings.get('DEFAULT_DEDUP_PIPELINE', '')
                pipelines = self.crawler.settings.get('PIPELINES', [])
            except Exception:
                pass
            
        updated_configs = []
        
        if 'aioredis_filter' in current_filter_class or 'redis_filter' in current_filter_class:
            # 更新为内存过滤器
            if self.crawler and self.crawler.settings is not None:
                try:
                    self.crawler.settings.set('FILTER_CLASS', 'crawlo.filters.memory_filter.MemoryFilter')
                    updated_configs.append("filter")
                except Exception:
                    pass
        
        # 检查当前去重管道是否为Redis去重管道
        if 'redis_dedup_pipeline' in default_dedup_pipeline:
            # 更新为内存去重管道
            if self.crawler and self.crawler.settings is not None:
                try:
                    self.crawler.settings.set('DEFAULT_DEDUP_PIPELINE', 'crawlo.pipelines.memory_dedup_pipeline.MemoryDedupPipeline')
                    # 同时更新PIPELINES列表中的去重管道
                    if default_dedup_pipeline in pipelines:
                        # 找到并替换Redis去重管道为内存去重管道
                        index = pipelines.index(default_dedup_pipeline)
                        pipelines[index] = 'crawlo.pipelines.memory_dedup_pipeline.MemoryDedupPipeline'
                        self.crawler.settings.set('PIPELINES', pipelines)
                    updated_configs.append("dedup pipeline")
                except Exception:
                    pass
        
        # 合并日志输出
        if updated_configs:
            self.logger.debug(f"configuration updated: {', '.join(updated_configs)} -> memory mode")

    async def next_request(self):
        """Get next request"""
        if not self.queue_manager:
            return None
            
        try:
            request = await self.queue_manager.get()
            
            # 恢复 callback（从 Redis 队列取出时）
            if request:
                spider = getattr(self.crawler, 'spider', None)
                request = self.request_serializer.restore_after_deserialization(request, spider)
            
            return request
        except Exception as e:
            from crawlo.utils.error_handler import ErrorContext
            self.error_handler.handle_error(
                e, 
                context=ErrorContext(context="Failed to get next request"), 
                raise_error=False
            )
            return None

    async def enqueue_request(self, request):
        """Add request to queue"""
        # 修改调度器逻辑以正确处理Redis过滤器
        if not request.dont_filter:
            # 检查过滤器是否为Redis过滤器且有异步方法
            if hasattr(self.dupe_filter, 'requested_async'):
                # 对于Redis过滤器，使用异步方法
                is_duplicate = await self.dupe_filter.requested_async(request)
                if is_duplicate:
                    self.dupe_filter.log_stats(request)
                    return False
            else:
                # 对于其他过滤器，使用同步方法
                is_duplicate = await common_call(self.dupe_filter.requested, request)
                if is_duplicate:
                    self.dupe_filter.log_stats(request)
                    return False

        if not self.queue_manager:
            self.logger.error("Queue manager not initialized")
            return False

        set_request(request, self.priority)
        
        try:
            # 使用统一的队列接口
            success = await self.queue_manager.put(request, priority=getattr(request, 'priority', 0))
            
            if success:
                self.logger.debug(f"Request enqueued successfully: {request.url}")
            
            return success
        except Exception as e:
            from crawlo.utils.error_handler import ErrorContext
            self.error_handler.handle_error(
                e, 
                context=ErrorContext(context="Failed to enqueue request"), 
                raise_error=False
            )
            return False

    def idle(self) -> bool:
        """Check if queue is empty"""
        return len(self) == 0

    async def async_idle(self) -> bool:
        """Asynchronously check if queue is empty (more accurate)"""
        if not self.queue_manager:
            return True
        # 使用队列管理器的异步empty方法
        return await self.queue_manager.async_empty()

    async def close(self):
        """Close scheduler"""
        try:
            if isinstance(closed := getattr(self.dupe_filter, 'closed', None), Callable):
                await closed()
            
            if self.queue_manager:
                await self.queue_manager.close()
        except Exception as e:
            from crawlo.utils.error_handler import ErrorContext
            self.error_handler.handle_error(
                e, 
                context=ErrorContext(context="Failed to close scheduler"), 
                raise_error=False
            )

    async def ack_request(self, request):
        """确认请求处理完成"""
        # 由于我们不再使用处理队列，ack_request方法现在是一个空操作
        # 任务在从主队列取出时就已经被认为是完成的
        self.logger.debug(f"任务确认完成: {getattr(request, 'url', 'Unknown URL')}")

    def __len__(self):
        """Get queue size"""
        if not self.queue_manager:
            return 0
        # 返回同步的近似值，实际大小需要异步获取
        return 0 if self.queue_manager.empty() else 1


#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
动态加载功能测试示例
==================
测试新添加的动态加载功能是否符合预期
"""
import asyncio
from crawlo import Spider, Request


class TabControlTestSpider(Spider):
    """标签页控制测试爬虫"""
    name = 'tab_control_test_spider'
    
    custom_settings = {
        'DOWNLOADER_TYPE': 'playwright',
        'PLAYWRIGHT_BROWSER_TYPE': 'chromium',
        'PLAYWRIGHT_HEADLESS': True,
        'PLAYWRIGHT_SINGLE_BROWSER_MODE': True,
        'PLAYWRIGHT_MAX_PAGES_PER_BROWSER': 3,  # 限制最大页面数为3
    }
    
    def start_requests(self):
        # 测试多个URL，验证标签页控制功能
        urls = [
            'https://httpbin.org/html',  # 简单HTML页面
            'https://httpbin.org/json',  # JSON页面
            'https://httpbin.org/xml',   # XML页面
            'https://httpbin.org/robots.txt',  # 文本页面
            'https://httpbin.org/headers',  # 请求头页面
        ]
        
        for url in urls:
            yield Request(url=url, callback=self.parse)
    
    def parse(self, response):
        """解析响应"""
        print(f"成功下载页面: {response.url}")
        print(f"响应状态码: {response.status_code}")
        print(f"响应内容长度: {len(response.text)}")
        yield {'url': response.url, 'status': response.status_code}


class PaginationTestSpider(Spider):
    """翻页功能测试爬虫"""
    name = 'pagination_test_spider'
    
    custom_settings = {
        'DOWNLOADER_TYPE': 'playwright',
        'PLAYWRIGHT_BROWSER_TYPE': 'chromium',
        'PLAYWRIGHT_HEADLESS': True,
    }
    
    def start_requests(self):
        # 测试带有翻页操作的请求
        request = Request(
            url='https://httpbin.org/html',
            callback=self.parse_with_pagination
        ).set_dynamic_loader(
            True,
            {
                "pagination_actions": [
                    # 模拟鼠标滑动
                    {
                        "type": "scroll",
                        "params": {
                            "count": 2,
                            "distance": 300,
                            "delay": 500
                        }
                    }
                ]
            }
        )
        yield request
    
    def parse_with_pagination(self, response):
        """解析带翻页操作的响应"""
        print(f"成功下载页面: {response.url}")
        yield {'url': response.url, 'status': response.status_code}


# 运行测试
async def run_tests():
    """运行测试"""
    print("=== 动态加载功能测试 ===\n")
    
    # 测试1: 标签页控制功能
    print("1. 测试标签页控制功能:")
    print("   配置最大页面数为3，请求5个URL")
    print("   预期结果: 正常下载所有页面，但只创建3个标签页\n")
    
    # 测试2: 翻页功能
    print("2. 测试翻页功能:")
    print("   配置鼠标滑动翻页操作")
    print("   预期结果: 页面加载完成后执行滑动操作\n")
    
    print("请运行以下命令来执行测试:")
    print("crawlo crawl tab_control_test_spider")
    print("crawlo crawl pagination_test_spider")


if __name__ == '__main__':
    asyncio.run(run_tests())


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
测试CrawloConfig工厂模式兼容性
"""

import sys
import os
import traceback
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.config import CrawloConfig


def test_standalone_factory():
    """测试单机模式工厂函数"""
    print("测试单机模式工厂函数...")
    
    try:
        # 创建单机模式配置
        config = CrawloConfig.standalone(
            project_name='ofweek_standalone',
            concurrency=8,
            download_delay=1.0
        )
        
        print(f"配置创建成功")
        print(f"RUN_MODE: {config.get('RUN_MODE')}")
        print(f"QUEUE_TYPE: {config.get('QUEUE_TYPE')}")
        print(f"PROJECT_NAME: {config.get('PROJECT_NAME')}")
        print(f"CONCURRENCY: {config.get('CONCURRENCY')}")
        print(f"DOWNLOAD_DELAY: {config.get('DOWNLOAD_DELAY')}")
        
        # 验证配置是否正确
        assert config.get('RUN_MODE') == 'standalone'
        assert config.get('QUEUE_TYPE') == 'memory'
        assert config.get('PROJECT_NAME') == 'ofweek_standalone'
        assert config.get('CONCURRENCY') == 8
        assert config.get('DOWNLOAD_DELAY') == 1.0
        
        print("✅ 单机模式工厂函数测试通过")
        return True
        
    except Exception as e:
        print(f"❌ 单机模式工厂函数测试失败: {e}")
        traceback.print_exc()
        return False


def test_distributed_factory():
    """测试分布式模式工厂函数"""
    print("\n测试分布式模式工厂函数...")
    
    try:
        # 创建分布式模式配置
        config = CrawloConfig.distributed(
            redis_host='127.0.0.1',
            redis_port=6379,
            project_name='ofweek_distributed',
            concurrency=16,
            download_delay=0.5
        )
        
        print(f"配置创建成功")
        print(f"RUN_MODE: {config.get('RUN_MODE')}")
        print(f"QUEUE_TYPE: {config.get('QUEUE_TYPE')}")
        print(f"PROJECT_NAME: {config.get('PROJECT_NAME')}")
        print(f"CONCURRENCY: {config.get('CONCURRENCY')}")
        print(f"DOWNLOAD_DELAY: {config.get('DOWNLOAD_DELAY')}")
        print(f"REDIS_HOST: {config.get('REDIS_HOST')}")
        print(f"REDIS_PORT: {config.get('REDIS_PORT')}")
        
        # 验证配置是否正确
        assert config.get('RUN_MODE') == 'distributed'
        assert config.get('QUEUE_TYPE') == 'redis'
        assert config.get('PROJECT_NAME') == 'ofweek_distributed'
        assert config.get('CONCURRENCY') == 16
        assert config.get('DOWNLOAD_DELAY') == 0.5
        assert config.get('REDIS_HOST') == '127.0.0.1'
        assert config.get('REDIS_PORT') == 6379
        
        print("✅ 分布式模式工厂函数测试通过")
        return True
        
    except Exception as e:
        print(f"❌ 分布式模式工厂函数测试失败: {e}")
        traceback.print_exc()
        return False


def test_auto_factory():
    """测试自动模式工厂函数"""
    print("\n测试自动模式工厂函数...")
    
    try:
        # 创建自动模式配置
        config = CrawloConfig.auto(
            project_name='ofweek_auto',
            concurrency=12,
            download_delay=0.8
        )
        
        print(f"配置创建成功")
        print(f"RUN_MODE: {config.get('RUN_MODE')}")
        print(f"QUEUE_TYPE: {config.get('QUEUE_TYPE')}")
        print(f"PROJECT_NAME: {config.get('PROJECT_NAME')}")
        print(f"CONCURRENCY: {config.get('CONCURRENCY')}")
        print(f"DOWNLOAD_DELAY: {config.get('DOWNLOAD_DELAY')}")
        
        # 验证配置是否正确
        assert config.get('RUN_MODE') == 'auto'
        assert config.get('QUEUE_TYPE') == 'auto'
        assert config.get('PROJECT_NAME') == 'ofweek_auto'
        assert config.get('CONCURRENCY') == 12
        assert config.get('DOWNLOAD_DELAY') == 0.8
        
        print("✅ 自动模式工厂函数测试通过")
        return True
        
    except Exception as e:
        print(f"❌ 自动模式工厂函数测试失败: {e}")
        traceback.print_exc()
        return False


def test_config_to_dict():
    """测试配置转换为字典"""
    print("\n测试配置转换为字典...")
    
    try:
        # 创建配置
        config = CrawloConfig.standalone(
            project_name='test_project',
            concurrency=4
        )
        
        # 转换为字典
        config_dict = config.to_dict()
        
        print(f"字典转换成功")
        print(f"字典键数量: {len(config_dict)}")
        
        # 验证关键配置项
        assert 'RUN_MODE' in config_dict
        assert 'QUEUE_TYPE' in config_dict
        assert 'PROJECT_NAME' in config_dict
        assert 'CONCURRENCY' in config_dict
        
        print("✅ 配置转换为字典测试通过")
        return True
        
    except Exception as e:
        print(f"❌ 配置转换为字典测试失败: {e}")
        traceback.print_exc()
        return False


def main():
    """主函数"""
    print("开始测试CrawloConfig工厂模式兼容性...")
    print("=" * 50)
    
    tests = [
        test_standalone_factory,
        test_distributed_factory,
        test_auto_factory,
        test_config_to_dict,
    ]
    
    passed = 0
    total = len(tests)
    
    for test_func in tests:
        try:
            if test_func():
                passed += 1
                print(f"✓ {test_func.__name__} 通过")
            else:
                print(f"✗ {test_func.__name__} 失败")
        except Exception as e:
            print(f"✗ {test_func.__name__} 异常: {e}")
        print()
    
    print("=" * 50)
    print(f"测试结果: {passed}/{total} 通过")
    
    if passed == total:
        print("所有测试通过！CrawloConfig工厂模式兼容性正常。")
        return 0
    else:
        print("部分测试失败，请检查实现。")
        return 1


if __name__ == "__main__":
    exit_code = main()
    exit(exit_code)


#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
Crawlo过滤器模块
================
提供多种请求去重过滤器实现。

过滤器类型:
- MemoryFilter: 基于内存的高效去重，适合单机模式
- AioRedisFilter: 基于Redis的分布式去重，适合分布式模式
- MemoryFileFilter: 内存+文件持久化，适合需要重启恢复的场景

核心接口:
- BaseFilter: 所有过滤器的基类
- requested(): 检查请求是否重复的主要方法
"""
from abc import ABC, abstractmethod
from typing import Optional

from crawlo.utils.fingerprint import FingerprintGenerator


class BaseFilter(ABC):
    """
    请求去重过滤器基类
    
    提供统一的去重接口和统计功能。
    所有过滤器实现都应该继承此类。
    """

    def __init__(self, logger, stats, debug: bool = False):
        """
        初始化过滤器
        
        :param logger: 日志器实例
        :param stats: 统计信息存储
        :param debug: 是否启用调试日志
        """
        self.logger = logger
        self.stats = stats
        self.debug = debug
        self._request_count = 0
        self._duplicate_count = 0

    @classmethod
    def create_instance(cls, *args, **kwargs) -> 'BaseFilter':
        return cls(*args, **kwargs)

    def _get_fingerprint(self, request) -> str:
        """
        获取请求指纹（内部辅助方法）
        
        使用统一的 FingerprintGenerator 生成请求指纹。
        子类可以直接调用此方法，避免重复实现。
        
        :param request: 请求对象
        :return: 请求指纹字符串
        """
        return FingerprintGenerator.request_fingerprint(
            request.method,
            request.url,
            request.body or b'',
            dict(request.headers) if hasattr(request, 'headers') else {}
        )

    def requested(self, request) -> bool:
        """
        检查请求是否重复（主要接口）
        
        :param request: 请求对象
        :return: True 表示重复，False 表示新请求
        """
        self._request_count += 1
        fp = self._get_fingerprint(request)
        
        if fp in self:
            self._duplicate_count += 1
            self.log_stats(request)
            return True
            
        self.add_fingerprint(fp)
        return False

    @abstractmethod
    def add_fingerprint(self, fp: str) -> None:
        """
        添加请求指纹（子类必须实现）
        
        :param fp: 请求指纹字符串
        """
        pass
    
    @abstractmethod
    def __contains__(self, item: str) -> bool:
        """
        检查指纹是否存在（支持 in 操作符）
        
        :param item: 要检查的指纹
        :return: 是否已存在
        """
        pass

    def log_stats(self, request) -> None:
        """
        记录统计信息
        
        :param request: 重复的请求对象
        """
        if self.debug:
            self.logger.debug(f'过滤重复请求: {request}')
        self.stats.inc_value(f'{self}/filtered_count')
    
    def get_stats(self) -> dict:
        """
        获取过滤器统计信息
        
        :return: 统计信息字典
        """
        return {
            'total_requests': self._request_count,
            'duplicate_requests': self._duplicate_count,
            'unique_requests': self._request_count - self._duplicate_count,
            'duplicate_rate': f"{self._duplicate_count / max(1, self._request_count) * 100:.2f}%"
        }
    
    def reset_stats(self) -> None:
        """重置统计信息"""
        self._request_count = 0
        self._duplicate_count = 0
    
    def close(self) -> None:
        """关闭过滤器并清理资源"""
        pass

    def __str__(self) -> str:
        return f'{self.__class__.__name__}'


# 导出所有可用的过滤器
__all__ = ['BaseFilter']

# 动态导入具体实现
try:
    from .memory_filter import MemoryFilter, MemoryFileFilter
    __all__.extend(['MemoryFilter', 'MemoryFileFilter'])
except ImportError:
    MemoryFilter = None
    MemoryFileFilter = None

try:
    from .aioredis_filter import AioRedisFilter
    __all__.append('AioRedisFilter')
except ImportError:
    AioRedisFilter = None

# 提供便捷的过滤器映射
FILTER_MAP = {
    'memory': MemoryFilter,
    'memory_file': MemoryFileFilter,
    'redis': AioRedisFilter,
    'aioredis': AioRedisFilter,  # 别名
}

# 过滤掉不可用的过滤器
FILTER_MAP = {k: v for k, v in FILTER_MAP.items() if v is not None}

def get_filter_class(name: str):
    """根据名称获取过滤器类"""
    if name in FILTER_MAP:
        return FILTER_MAP[name]
    raise ValueError(f"未知的过滤器类型: {name}。可用类型: {list(FILTER_MAP.keys())}")


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
分布式采集功能验证脚本
验证Crawlo框架的分布式采集功能是否正常工作
"""

import redis
import json
import os
import sys

# 添加项目根目录到 Python 路径
project_root = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, project_root)


def verify_distributed_functionality():
    """验证分布式采集功能"""
    print("=== Crawlo分布式采集功能验证 ===\n")

    # 1. 连接Redis
    try:
        r = redis.Redis(host='localhost', port=6379, db=2, decode_responses=False)
        r.ping()
        print("✓ Redis连接成功")
    except Exception as e:
        print(f"✗ Redis连接失败: {e}")
        return False

    # 2. 检查项目配置
    try:
        with open('../examples/ofweek_distributed/crawlo.cfg', 'r') as f:
            config_content = f.read()
            if 'ofweek_distributed.settings' in config_content:
                print("✓ 项目配置文件正确")
            else:
                print("✗ 项目配置文件不正确")
                return False
    except Exception as e:
        print(f"✗ 无法读取配置文件: {e}")
        return False

    # 3. 检查设置文件
    try:
        with open('../examples/ofweek_distributed/ofweek_distributed/settings.py', 'r') as f:
            settings_content = f.read()
            checks = [
                ('RUN_MODE = \'distributed\'', '运行模式设置为分布式'),
                ('QUEUE_TYPE = \'redis\'', '队列类型设置为Redis'),
                ('FILTER_CLASS = \'crawlo.filters.aioredis_filter.AioRedisFilter\'', '过滤器设置为Redis过滤器'),
                ('REDIS_HOST = \'127.0.0.1\'', 'Redis主机配置正确'),
            ]

            all_passed = True
            for check, description in checks:
                if check in settings_content:
                    print(f"✓ {description}")
                else:
                    print(f"✗ {description}")
                    all_passed = False

            if not all_passed:
                return False
    except Exception as e:
        print(f"✗ 无法读取设置文件: {e}")
        return False

    # 4. 检查Redis中的数据
    try:
        # 检查请求去重指纹
        request_fingerprints = r.scard("crawlo:ofweek_distributed:filter:fingerprint")
        print(f"✓ 请求去重指纹数量: {request_fingerprints}")

        # 检查数据项去重指纹
        item_fingerprints = r.scard("crawlo:ofweek_distributed:item:fingerprint")
        print(f"✓ 数据项去重指纹数量: {item_fingerprints}")

        # 检查请求队列
        queue_size = r.zcard("crawlo:ofweek_distributed:queue:requests")
        print(f"✓ 请求队列大小: {queue_size}")

        # 验证数据是否存在
        if request_fingerprints > 0 and item_fingerprints > 0:
            print("✓ Redis中存在分布式采集数据")
        else:
            print("⚠ Redis中分布式采集数据为空")

    except Exception as e:
        print(f"✗ Redis数据检查失败: {e}")
        return False

    # 5. 检查输出文件
    try:
        import glob
        json_files = glob.glob("output/*.json")
        if json_files:
            latest_file = max(json_files, key=os.path.getctime)
            file_size = os.path.getsize(latest_file)
            print(f"✓ 输出文件存在: {latest_file} ({file_size} bytes)")
        else:
            print("⚠ 未找到输出文件")
    except Exception as e:
        print(f"✗ 输出文件检查失败: {e}")

    print("\n=== 验证结果 ===")
    print("✓ Crawlo分布式采集功能正常工作!")
    print("  - Redis连接正常")
    print("  - 分布式配置正确")
    print("  - Redis数据存储正常")
    print("  - 采集任务执行正常")

    return True


if __name__ == '__main__':
    verify_distributed_functionality()



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
ProxyMiddleware 重构测试文件
用于测试重构后的代理中间件功能，特别是修复的重复逻辑
"""

import asyncio
import unittest
from unittest.mock import Mock, patch

from crawlo.middleware.proxy import ProxyMiddleware, Proxy
from crawlo.exceptions import NotConfiguredError
from crawlo.settings.setting_manager import SettingManager


class TestProxyMiddlewareRefactored(unittest.TestCase):
    """ProxyMiddleware 重构测试类"""

    def setUp(self):
        """测试前准备"""
        # 创建设置管理器
        self.settings = SettingManager()
        
        # 创建爬虫模拟对象
        self.crawler = Mock()
        self.crawler.settings = self.settings

    def test_parse_proxy_data_with_string(self):
        """测试解析字符串代理数据"""
        middleware = ProxyMiddleware(
            settings=self.settings,
            log_level='INFO'
        )
        
        # 测试有效的HTTP代理URL
        result = middleware._parse_proxy_data("http://proxy.example.com:8080")
        self.assertEqual(result, ["http://proxy.example.com:8080"])
        
        # 测试有效的HTTPS代理URL
        result = middleware._parse_proxy_data("https://proxy.example.com:8080")
        self.assertEqual(result, ["https://proxy.example.com:8080"])
        
        # 测试无效的代理URL
        result = middleware._parse_proxy_data("invalid-proxy")
        self.assertEqual(result, [])

    def test_parse_proxy_data_with_dict(self):
        """测试解析字典代理数据"""
        middleware = ProxyMiddleware(
            settings=self.settings,
            log_level='INFO'
        )
        
        # 测试包含字符串代理的字典
        proxy_data = {
            "proxy": "http://proxy1.example.com:8080"
        }
        result = middleware._parse_proxy_data(proxy_data)
        self.assertEqual(result, ["http://proxy1.example.com:8080"])
        
        # 测试包含列表代理的字典
        proxy_data = {
            "proxies": [
                "http://proxy1.example.com:8080",
                "https://proxy2.example.com:8080"
            ]
        }
        result = middleware._parse_proxy_data(proxy_data)
        self.assertEqual(result, [
            "http://proxy1.example.com:8080",
            "https://proxy2.example.com:8080"
        ])
        
        # 测试混合数据
        proxy_data = {
            "proxy": "http://proxy1.example.com:8080",
            "proxies": [
                "https://proxy2.example.com:8080",
                "invalid-proxy"
            ]
        }
        result = middleware._parse_proxy_data(proxy_data)
        self.assertEqual(result, [
            "http://proxy1.example.com:8080",
            "https://proxy2.example.com:8080"
        ])

    def test_get_healthy_proxies(self):
        """测试获取健康代理"""
        middleware = ProxyMiddleware(
            settings=self.settings,
            log_level='INFO'
        )
        
        # 创建测试代理
        proxy1 = Proxy("http://proxy1.example.com:8080")
        proxy2 = Proxy("http://proxy2.example.com:8080")
        proxy3 = Proxy("http://proxy3.example.com:8080")
        
        # 设置代理池
        middleware._proxy_pool = [proxy1, proxy2, proxy3]
        
        # 所有代理都是健康的
        healthy_proxies = middleware._get_healthy_proxies()
        self.assertEqual(len(healthy_proxies), 3)
        
        # 标记一个代理为不健康
        proxy2.is_healthy = False
        healthy_proxies = middleware._get_healthy_proxies()
        self.assertEqual(len(healthy_proxies), 2)
        self.assertIn(proxy1, healthy_proxies)
        self.assertNotIn(proxy2, healthy_proxies)
        self.assertIn(proxy3, healthy_proxies)
        
        # 标记一个代理成功率低于阈值
        proxy3.mark_failure()
        proxy3.mark_failure()
        proxy3.mark_failure()
        proxy3.mark_failure()  # 4次失败，0次成功，成功率=0 < 0.5(默认阈值)
        healthy_proxies = middleware._get_healthy_proxies()
        self.assertEqual(len(healthy_proxies), 1)
        self.assertIn(proxy1, healthy_proxies)
        self.assertNotIn(proxy3, healthy_proxies)

    @patch('crawlo.utils.log.get_logger')
    def test_update_proxy_pool_with_parsed_data(self, mock_get_logger):
        """测试使用解析后的代理数据更新代理池"""
        # 不再需要 PROXY_ENABLED，只要配置了 PROXY_API_URL 就会启用
        self.settings.set('PROXY_API_URL', 'http://proxy-api.example.com')
        self.settings.set('LOG_LEVEL', 'INFO')
        
        mock_get_logger.return_value = Mock()
        
        middleware = ProxyMiddleware.create_instance(self.crawler)
        
        # 测试解析字符串代理数据
        new_proxies = middleware._parse_proxy_data("http://proxy1.example.com:8080")
        self.assertEqual(new_proxies, ["http://proxy1.example.com:8080"])
        
        # 测试解析字典代理数据
        proxy_data = {
            "proxies": [
                "http://proxy1.example.com:8080",
                "https://proxy2.example.com:8080",
                "http://proxy3.example.com:8080"
            ]
        }
        new_proxies = middleware._parse_proxy_data(proxy_data)
        self.assertEqual(new_proxies, [
            "http://proxy1.example.com:8080",
            "https://proxy2.example.com:8080",
            "http://proxy3.example.com:8080"
        ])

    def test_get_healthy_proxy_with_refactored_logic(self):
        """测试使用重构后的逻辑获取健康代理"""
        middleware = ProxyMiddleware(
            settings=self.settings,
            log_level='INFO'
        )
        
        # 创建测试代理
        proxy1 = Proxy("http://proxy1.example.com:8080")
        proxy2 = Proxy("http://proxy2.example.com:8080")
        
        # 设置代理池
        middleware._proxy_pool = [proxy1, proxy2]
        middleware._current_proxy_index = 0
        
        # 获取第一个健康代理（由于轮询逻辑，第一次调用会得到索引1的代理）
        healthy_proxy = asyncio.run(middleware._get_healthy_proxy())
        self.assertEqual(healthy_proxy.proxy_str, proxy2.proxy_str)
        
        # 获取第二个健康代理（轮询回到索引0）
        healthy_proxy = asyncio.run(middleware._get_healthy_proxy())
        self.assertEqual(healthy_proxy.proxy_str, proxy1.proxy_str)
        
        # 再次获取第一个健康代理（轮询到索引1）
        healthy_proxy = asyncio.run(middleware._get_healthy_proxy())
        self.assertEqual(healthy_proxy.proxy_str, proxy2.proxy_str)

    def test_proxy_middleware_initialization(self):
        """测试代理中间件初始化"""
        # 不再需要 PROXY_ENABLED，只要配置了 PROXY_API_URL 就会启用
        self.settings.set('PROXY_API_URL', 'http://test-proxy-api.com')
        middleware = ProxyMiddleware(self.settings, "DEBUG")
        self.assertIsInstance(middleware, ProxyMiddleware)
        self.assertTrue(middleware.enabled)
        self.assertEqual(middleware.api_url, 'http://test-proxy-api.com')

    def test_proxy_middleware_enabled_with_api_url(self):
        """测试配置了代理API URL时中间件启用"""
        # 不再需要 PROXY_ENABLED，只要配置了 PROXY_API_URL 就会启用
        self.settings.set('PROXY_API_URL', 'http://test-proxy-api.com')
        middleware = ProxyMiddleware(self.settings, "DEBUG")
        self.assertTrue(middleware.enabled)
        self.assertEqual(middleware.api_url, 'http://test-proxy-api.com')

    def test_proxy_middleware_disabled_without_api_url(self):
        """测试未配置代理API URL时中间件禁用"""
        # 不再需要 PROXY_ENABLED，只要不配置 PROXY_API_URL 就会禁用
        self.settings.set('PROXY_API_URL', None)
        middleware = ProxyMiddleware(self.settings, "DEBUG")
        self.assertFalse(middleware.enabled)

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
综合测试SpiderLoader功能
"""

import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.utils.spider_loader import SpiderLoader
from crawlo.crawler import CrawlerProcess
from crawlo.settings.setting_manager import SettingManager


def test_spider_loader_comprehensive():
    """综合测试SpiderLoader功能"""
    print("综合测试SpiderLoader功能...")
    
    # 1. 测试基本的SpiderLoader功能
    print("\n1. 测试基本的SpiderLoader功能")
    settings = SettingManager({
        'SPIDER_MODULES': ['tests.test_spiders'],
        'SPIDER_LOADER_WARN_ONLY': True
    })
    
    loader = SpiderLoader.from_settings(settings)
    spider_names = loader.list()
    print(f"  发现的爬虫: {spider_names}")
    
    if spider_names:
        spider_name = spider_names[0]
        spider_class = loader.load(spider_name)
        print(f"  成功加载爬虫: {spider_name} -> {spider_class}")
    
    # 2. 测试CrawlerProcess与SPIDER_MODULES的集成
    print("\n2. 测试CrawlerProcess与SPIDER_MODULES的集成")
    process = CrawlerProcess(settings=settings)
    process_spider_names = process.get_spider_names()
    print(f"  CrawlerProcess发现的爬虫: {process_spider_names}")
    
    is_registered = process.is_spider_registered('test_spider')
    print(f"  爬虫'test_spider'是否已注册: {is_registered}")
    
    spider_class = process.get_spider_class('test_spider')
    print(f"  爬虫'test_spider'的类: {spider_class}")
    
    # 3. 测试接口规范
    print("\n3. 测试接口规范")
    # 检查SpiderLoader是否实现了ISpiderLoader接口所需的方法
    from crawlo.interfaces import ISpiderLoader
    # 由于ISpiderLoader是Protocol，我们不能直接使用isinstance检查
    # 而是检查是否实现了所需的方法
    required_methods = ['load', 'list', 'find_by_request']
    implements_interface = all(hasattr(loader, method) for method in required_methods)
    print(f"  SpiderLoader是否实现了ISpiderLoader接口: {implements_interface}")
    
    # 4. 测试方法存在性
    print("\n4. 测试方法存在性")
    required_methods = ['load', 'list', 'find_by_request', 'get_all']
    for method in required_methods:
        has_method = hasattr(loader, method)
        print(f"  SpiderLoader是否有{method}方法: {has_method}")
    
    print("\n综合测试完成!")


if __name__ == '__main__':
    test_spider_loader_comprehensive()


#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
配置管理模块
===========
统一的配置管理接口，整合了通用配置工具、环境变量管理和大规模爬虫配置。

本模块包含：
1. ConfigUtils - 通用配置工具类
2. EnvConfigManager - 环境变量配置管理器
3. LargeScaleConfig - 大规模爬虫配置类
4. 便捷函数 - 快速访问常用配置功能
"""

import os
import re
from typing import Any, Dict, List, Optional, Union


# ============================================================================
# 第一部分：通用配置工具
# ============================================================================

class ConfigUtils:
    """通用配置工具类"""
    
    @staticmethod
    def get_config_value(
        config_sources: List[Union[Dict, Any]], 
        key: str, 
        default: Any = None,
        value_type: type = str
    ) -> Any:
        """
        从多个配置源中获取配置值
        
        Args:
            config_sources: 配置源列表，按优先级排序
            key: 配置键名
            default: 默认值
            value_type: 值类型
            
        Returns:
            配置值或默认值
        """
        for config_source in config_sources:
            if not config_source:
                continue
                
            # 获取配置值
            value = None
            if hasattr(config_source, 'get'):
                value = config_source.get(key)
            elif hasattr(config_source, key):
                value = getattr(config_source, key)
            else:
                continue
                
            if value is not None:
                # 类型转换
                try:
                    if value_type == bool:
                        if isinstance(value, str):
                            return value.lower() in ('1', 'true', 'yes', 'on')
                        return bool(value)
                    elif value_type == int:
                        return int(value)
                    elif value_type == float:
                        return float(value)
                    else:
                        return value_type(value)
                except (ValueError, TypeError):
                    continue
        
        return default
    
    @staticmethod
    def has_config_prefix(config_source: Union[Dict, Any], prefix: str) -> bool:
        """
        检查配置源是否包含指定前缀的配置项
        
        Args:
            config_source: 配置源
            prefix: 前缀
            
        Returns:
            是否包含指定前缀的配置项
        """
        if not config_source:
            return False
            
        if hasattr(config_source, 'keys'):
            return any(key.startswith(prefix) for key in config_source.keys())
        elif hasattr(config_source, '__dict__'):
            return any(key.startswith(prefix) for key in config_source.__dict__.keys())
        else:
            return any(key.startswith(prefix) for key in dir(config_source))
    
    @staticmethod
    def merge_config_sources(config_sources: List[Union[Dict, Any]]) -> Dict[str, Any]:
        """
        合并多个配置源，后面的配置源优先级更高
        
        Args:
            config_sources: 配置源列表
            
        Returns:
            合并后的配置字典
        """
        merged_config = {}
        
        for config_source in config_sources:
            if not config_source:
                continue
                
            if hasattr(config_source, 'keys'):
                # 字典类型配置源
                for key, value in config_source.items():
                    if key.isupper():  # 只合并大写的配置项
                        merged_config[key] = value
            elif hasattr(config_source, '__dict__'):
                # 对象类型配置源
                for key, value in config_source.__dict__.items():
                    if key.isupper():
                        merged_config[key] = value
            else:
                # 其他类型配置源
                for key in dir(config_source):
                    if key.isupper():
                        merged_config[key] = getattr(config_source, key)
        
        return merged_config


# ============================================================================
# 第二部分：环境变量配置管理
# ============================================================================

class EnvConfigManager:
    """环境变量配置管理器"""
    
    @staticmethod
    def get_env_var(var_name: str, default: Any = None, var_type: type = str) -> Any:
        """
        获取环境变量值
        
        Args:
            var_name: 环境变量名称
            default: 默认值
            var_type: 变量类型 (str, int, float, bool)
            
        Returns:
            环境变量值或默认值
        """
        value = os.getenv(var_name)
        if value is None:
            return default
        
        try:
            if var_type == bool:
                return value.lower() in ('1', 'true', 'yes', 'on')
            elif var_type == int:
                return int(value)
            elif var_type == float:
                return float(value)
            else:
                return value
        except (ValueError, TypeError):
            return default
    
    @staticmethod
    def get_redis_config() -> dict:
        """
        获取 Redis 配置
        
        Returns:
            Redis 配置字典
        """
        return {
            'REDIS_HOST': EnvConfigManager.get_env_var('CRAWLO_REDIS_HOST', '127.0.0.1', str),
            'REDIS_PORT': EnvConfigManager.get_env_var('CRAWLO_REDIS_PORT', 6379, int),
            'REDIS_PASSWORD': EnvConfigManager.get_env_var('CRAWLO_REDIS_PASSWORD', '', str),
            'REDIS_DB': EnvConfigManager.get_env_var('CRAWLO_REDIS_DB', 0, int),
        }
    
    @staticmethod
    def get_runtime_config() -> dict:
        """
        获取运行时配置
        
        Returns:
            运行时配置字典
        """
        return {
            'CRAWLO_MODE': EnvConfigManager.get_env_var('CRAWLO_MODE', 'standalone', str),
            'PROJECT_NAME': EnvConfigManager.get_env_var('CRAWLO_PROJECT_NAME', 'crawlo', str),
            'CONCURRENCY': EnvConfigManager.get_env_var('CRAWLO_CONCURRENCY', 8, int),
        }

    @staticmethod
    def get_version() -> str:
        """
        获取框架版本号
        
        Returns:
            框架版本号字符串
        """
        # 获取版本文件路径
        version_file = os.path.join(os.path.dirname(__file__), '..', '__version__.py')
        default_version = '1.0.0'
        
        if os.path.exists(version_file):
            try:
                with open(version_file, 'r', encoding='utf-8') as f:
                    content = f.read()
                    # 使用正则表达式提取版本号
                    version_match = re.search(r"__version__\s*=\s*['\"]([^'\"]*)['\"]", content)
                    if version_match:
                        return version_match.group(1)
            except Exception:
                # 如果读取失败，使用默认版本号
                pass
        
        return default_version


# ============================================================================
# 第三部分：大规模爬虫配置
# ============================================================================

class LargeScaleConfig:
    """大规模爬虫配置类"""
    
    @staticmethod
    def conservative_config(concurrency: int = 8) -> Dict[str, Any]:
        """
        保守配置 - 适用于资源有限的环境
        
        特点：
        - 较小的队列容量
        - 较低的并发数
        - 较长的延迟
        """
        from crawlo.utils.queue_helper import QueueHelper
        
        config = QueueHelper.use_redis_queue(
            queue_name="crawlo:conservative",
            max_retries=3,
            timeout=300
        )
        
        config.update({
            # 并发控制
            'CONCURRENCY': concurrency,
            'SCHEDULER_MAX_QUEUE_SIZE': concurrency * 10,
            'MAX_RUNNING_SPIDERS': 1,
            
            # 请求控制
            'DOWNLOAD_DELAY': 0.2,
            'RANDOMNESS': True,
            'RANDOM_RANGE': (0.8, 1.5),
            
            # 内存控制
            'DOWNLOAD_MAXSIZE': 5 * 1024 * 1024,  # 5MB
            'CONNECTION_POOL_LIMIT': concurrency * 2,
            
            # 重试策略
            'MAX_RETRY_TIMES': 2,
            
            # 使用增强引擎
            'ENGINE_CLASS': 'crawlo.core.engine.Engine'
        })
        
        return config
    
    @staticmethod
    def balanced_config(concurrency: int = 16) -> Dict[str, Any]:
        """
        平衡配置 - 适用于一般生产环境
        
        特点：
        - 中等的队列容量
        - 平衡的并发数
        - 适中的延迟
        """
        from crawlo.utils.queue_helper import QueueHelper
        
        config = QueueHelper.use_redis_queue(
            queue_name="crawlo:balanced",
            max_retries=5,
            timeout=600
        )
        
        config.update({
            # 并发控制
            'CONCURRENCY': concurrency,
            'SCHEDULER_MAX_QUEUE_SIZE': concurrency * 15,
            'MAX_RUNNING_SPIDERS': 2,
            
            # 请求控制
            'DOWNLOAD_DELAY': 0.1,
            'RANDOMNESS': True,
            'RANDOM_RANGE': (0.5, 1.2),
            
            # 内存控制
            'DOWNLOAD_MAXSIZE': 10 * 1024 * 1024,  # 10MB
            'CONNECTION_POOL_LIMIT': concurrency * 3,
            
            # 重试策略
            'MAX_RETRY_TIMES': 3,
            
            # 使用增强引擎
            'ENGINE_CLASS': 'crawlo.core.engine.Engine'
        })
        
        return config
    
    @staticmethod
    def aggressive_config(concurrency: int = 32) -> Dict[str, Any]:
        """
        激进配置 - 适用于高性能环境
        
        特点：
        - 大的队列容量
        - 高并发数
        - 较短的延迟
        """
        from crawlo.utils.queue_helper import QueueHelper
        
        config = QueueHelper.use_redis_queue(
            queue_name="crawlo:aggressive",
            max_retries=10,
            timeout=900
        )
        
        config.update({
            # 并发控制
            'CONCURRENCY': concurrency,
            'SCHEDULER_MAX_QUEUE_SIZE': concurrency * 20,
            'MAX_RUNNING_SPIDERS': 3,
            
            # 请求控制
            'DOWNLOAD_DELAY': 0.05,
            'RANDOMNESS': True,
            'RANDOM_RANGE': (0.3, 1.0),
            
            # 内存控制
            'DOWNLOAD_MAXSIZE': 20 * 1024 * 1024,  # 20MB
            'CONNECTION_POOL_LIMIT': concurrency * 4,
            
            # 重试策略
            'MAX_RETRY_TIMES': 5,
            
            # 使用增强引擎
            'ENGINE_CLASS': 'crawlo.core.engine.Engine'
        })
        
        return config
    
    @staticmethod
    def memory_optimized_config(concurrency: int = 12) -> Dict[str, Any]:
        """
        内存优化配置 - 适用于大规模但内存受限的场景
        
        特点：
        - 小队列，快速流转
        - 严格的内存控制
        - 使用Redis减少内存压力
        """
        from crawlo.utils.queue_helper import QueueHelper
        
        config = QueueHelper.use_redis_queue(
            queue_name="crawlo:memory_optimized",
            max_retries=3,
            timeout=300
        )
        
        config.update({
            # 并发控制
            'CONCURRENCY': concurrency,
            'SCHEDULER_MAX_QUEUE_SIZE': concurrency * 5,
            'MAX_RUNNING_SPIDERS': 1,
            
            # 请求控制
            'DOWNLOAD_DELAY': 0.1,
            'RANDOMNESS': False,
            
            # 严格的内存控制
            'DOWNLOAD_MAXSIZE': 2 * 1024 * 1024,  # 2MB
            'DOWNLOAD_WARN_SIZE': 512 * 1024,     # 512KB
            'CONNECTION_POOL_LIMIT': concurrency,
            
            # 重试策略
            'MAX_RETRY_TIMES': 2,
            
            # 使用增强引擎
            'ENGINE_CLASS': 'crawlo.core.engine.Engine'
        })
        
        return config


def apply_large_scale_config(
    settings_dict: Dict[str, Any], 
    config_type: str = "balanced", 
    concurrency: Optional[int] = None
):
    """
    应用大规模配置
    
    Args:
        settings_dict: 设置字典
        config_type: 配置类型 ("conservative", "balanced", "aggressive", "memory_optimized")
        concurrency: 并发数（可选，不指定则使用默认值）
    """
    config_map = {
        "conservative": LargeScaleConfig.conservative_config,
        "balanced": LargeScaleConfig.balanced_config,
        "aggressive": LargeScaleConfig.aggressive_config,
        "memory_optimized": LargeScaleConfig.memory_optimized_config
    }
    
    if config_type not in config_map:
        raise ValueError(f"不支持的配置类型: {config_type}")
    
    if concurrency:
        config = config_map[config_type](concurrency)
    else:
        config = config_map[config_type]()
    
    settings_dict.update(config)
    
    return config


# 导出所有公共API
__all__ = [
    'ConfigUtils',
    'EnvConfigManager',
    'LargeScaleConfig',
    'apply_large_scale_config',
]



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
队列配置辅助工具
为用户提供简洁的队列配置接口
"""
from typing import Dict, Any, Optional


class QueueHelper:
    """队列配置辅助类"""
    
    @staticmethod
    def use_memory_queue(max_size: int = 2000) -> Dict[str, Any]:
        """
        配置使用内存队列
        
        Args:
            max_size: 队列最大容量
            
        Returns:
            配置字典
        """
        return {
            'QUEUE_TYPE': 'memory',
            'SCHEDULER_MAX_QUEUE_SIZE': max_size,
        }
    
    @staticmethod
    def use_redis_queue(
        host: str = "127.0.0.1",
        port: int = 6379,
        password: Optional[str] = None,
        db: int = 0,
        queue_name: str = "crawlo:requests",
        max_retries: int = 3,
        timeout: int = 300
    ) -> Dict[str, Any]:
        """
        配置使用 Redis 分布式队列
        
        Args:
            host: Redis 主机地址
            port: Redis 端口
            password: Redis 密码（可选）
            db: Redis 数据库编号
            queue_name: 队列名称
            max_retries: 最大重试次数
            timeout: 操作超时时间（秒）
            
        Returns:
            配置字典
        """
        if password:
            redis_url = f"redis://:{password}@{host}:{port}/{db}"
        else:
            redis_url = f"redis://{host}:{port}/{db}"
        
        return {
            'QUEUE_TYPE': 'redis',
            'REDIS_URL': redis_url,
            'REDIS_HOST': host,
            'REDIS_PORT': port,
            'REDIS_PASSWORD': password or '',
            'REDIS_DB': db,
            'SCHEDULER_QUEUE_NAME': queue_name,
            'QUEUE_MAX_RETRIES': max_retries,
            'QUEUE_TIMEOUT': timeout,
        }
    
    @staticmethod
    def auto_queue(
        redis_fallback: bool = True,
        memory_max_size: int = 2000,
        **redis_kwargs
    ) -> Dict[str, Any]:
        """
        配置自动选择队列类型
        
        Args:
            redis_fallback: Redis 不可用时是否回退到内存队列
            memory_max_size: 内存队列最大容量
            **redis_kwargs: Redis 配置参数
            
        Returns:
            配置字典
        """
        config = {
            'QUEUE_TYPE': 'auto',
            'SCHEDULER_MAX_QUEUE_SIZE': memory_max_size,
        }
        
        # 添加 Redis 配置（用于自动检测）
        if redis_kwargs:
            redis_config = QueueHelper.use_redis_queue(**redis_kwargs)
            config.update(redis_config)
            config['QUEUE_TYPE'] = 'auto'  # 确保是自动模式
        
        return config


# 预定义的常用配置
class QueuePresets:
    """预定义的队列配置"""
    
    # 开发环境：使用内存队列
    DEVELOPMENT = QueueHelper.use_memory_queue(max_size=1000)
    
    # 生产环境：使用 Redis 分布式队列
    PRODUCTION = QueueHelper.use_redis_queue(
        host="127.0.0.1",
        port=6379,
        queue_name="crawlo:production",
        max_retries=5,
        timeout=600
    )
    
    # 测试环境：自动选择，Redis 不可用时使用内存队列
    TESTING = QueueHelper.auto_queue(
        redis_fallback=True,
        memory_max_size=500,
        host="127.0.0.1",
        port=6379,
        queue_name="crawlo:testing"
    )
    
    # 高性能环境：Redis 集群
    HIGH_PERFORMANCE = QueueHelper.use_redis_queue(
        host="redis-cluster.example.com",
        port=6379,
        queue_name="crawlo:cluster",
        max_retries=10,
        timeout=300
    )


def apply_queue_config(settings_dict: Dict[str, Any], config: Dict[str, Any]) -> None:
    """
    将队列配置应用到设置字典
    
    Args:
        settings_dict: 现有的设置字典
        config: 队列配置字典
    """
    settings_dict.update(config)


# 使用示例和文档
USAGE_EXAMPLES = """
# 使用示例：

# 1. 在 settings.py 中使用内存队列
from crawlo.utils.queue_helper import QueueHelper
apply_queue_config(locals(), QueueHelper.use_memory_queue())

# 2. 在 settings.py 中使用 Redis 队列
apply_queue_config(locals(), QueueHelper.use_redis_queue(
    host="redis.example.com",
    password="your_password"
))

# 3. 使用预定义配置
from crawlo.utils.queue_helper import QueuePresets
apply_queue_config(locals(), QueuePresets.PRODUCTION)

# 4. 自动选择队列类型
apply_queue_config(locals(), QueueHelper.auto_queue(
    host="127.0.0.1",
    port=6379
))

# 5. 直接在 settings 中配置
QUEUE_TYPE = 'auto'  # 'memory', 'redis', 'auto'
REDIS_URL = 'redis://127.0.0.1:6379/0'
SCHEDULER_MAX_QUEUE_SIZE = 2000
"""


# crawlo/cli.py
# !/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import argparse
from crawlo.commands import get_commands
from crawlo.utils.config_manager import EnvConfigManager


def main():
    # 获取框架版本号
    VERSION = EnvConfigManager.get_version()

    # 获取所有可用命令
    commands = get_commands()

    # 创建主解析器
    parser = argparse.ArgumentParser(
        description="Crawlo: A lightweight web crawler framework.",
        usage="crawlo <command> [options]",
        add_help=False  # 禁用默认帮助，我们自己处理
    )
    
    # 添加帮助参数
    parser.add_argument('-h', '--help', action='store_true', help='显示帮助信息')
    parser.add_argument('-v', '--version', action='store_true', help='显示版本信息')
    parser.add_argument('command', nargs='?', help='可用命令: ' + ', '.join(commands.keys()))
    
    # 解析已知参数
    args, unknown = parser.parse_known_args()

    # 处理版本参数
    if args.version:
        print(f"Crawlo {VERSION}")
        sys.exit(0)

    # 处理帮助参数
    if args.help or (args.command is None and not unknown):
        # 导入并运行帮助命令
        try:
            module = __import__(commands['help'], fromlist=['main'])
            sys.exit(module.main([]))
        except ImportError as e:
            print(f"Failed to load help command: {e}")
            sys.exit(1)
        except Exception as e:
            print(f"Help command failed: {e}")
            sys.exit(1)

    # 检查命令是否存在
    if args.command not in commands:
        print(f"Unknown command: {args.command}")
        print(f"Available commands: {', '.join(commands.keys())}")
        # 显示帮助信息
        try:
            module = __import__(commands['help'], fromlist=['main'])
            module.main([])
        except:
            pass
        sys.exit(1)

    # 动态导入并执行命令
    try:
        module = __import__(commands[args.command], fromlist=['main'])
        # 将未知参数传递给子命令
        sys.exit(module.main(unknown))
    except ImportError as e:
        print(f"Failed to load command '{args.command}': {e}")
        sys.exit(1)
    except Exception as e:
        print(f"Command '{args.command}' failed: {e}")
        sys.exit(1)


if __name__ == '__main__':
    main()


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
双重 crawlo 前缀问题修复简单测试脚本
用于验证 Redis 队列名称中双重 crawlo 前缀问题的修复，不依赖于实际的 Redis 连接
"""
import sys
import os
import asyncio
import traceback

# 添加项目根目录到路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

# 导入相关模块
from crawlo.queue.redis_priority_queue import RedisPriorityQueue


def test_redis_queue_naming():
    """测试 Redis 队列命名修复"""
    print("开始测试 Redis 队列命名修复...")
    print("=" * 50)
    
    test_cases = [
        {
            "name": "正常命名",
            "queue_name": "crawlo:test_project:queue:requests",
            "expected_queue": "crawlo:test_project:queue:requests",
            "expected_processing": "crawlo:test_project:queue:processing",
            "expected_failed": "crawlo:test_project:queue:failed"
        },
        {
            "name": "双重 crawlo 前缀",
            "queue_name": "crawlo:crawlo:queue:requests",
            "expected_queue": "crawlo:crawlo:queue:requests",  # 保持原始名称不变
            "expected_processing": "crawlo:crawlo:queue:processing",  # 保持一致的双重前缀
            "expected_failed": "crawlo:crawlo:queue:failed"
        },
        {
            "name": "三重 crawlo 前缀",
            "queue_name": "crawlo:crawlo:crawlo:queue:requests",
            "expected_queue": "crawlo:crawlo:crawlo:queue:requests",  # 保持原始名称不变
            "expected_processing": "crawlo:crawlo:crawlo:queue:processing",  # 保持一致的前缀
            "expected_failed": "crawlo:crawlo:crawlo:queue:failed"
        },
        {
            "name": "无 crawlo 前缀",
            "queue_name": "test_project:queue:requests",
            "expected_queue": "test_project:queue:requests",  # 保持原始名称不变
            "expected_processing": "test_project:queue:processing",
            "expected_failed": "test_project:queue:failed"
        }
    ]
    
    try:
        for i, test_case in enumerate(test_cases, 1):
            print(f"测试 {i}: {test_case['name']}")
            print(f"  输入队列名称: {test_case['queue_name']}")
            
            # 测试 RedisPriorityQueue 初始化
            try:
                queue = RedisPriorityQueue(
                    redis_url="redis://127.0.0.1:6379/15",
                    queue_name=test_case['queue_name'],
                    module_name="test_project"
                )
                
                print(f"  修复后队列名称: {queue.queue_name}")
                print(f"  修复后处理队列: {queue.processing_queue}")
                print(f"  修复后失败队列: {queue.failed_queue}")
                
                # 验证结果
                assert queue.queue_name == test_case['expected_queue'], \
                    f"队列名称不匹配: {queue.queue_name} != {test_case['expected_queue']}"
                assert queue.processing_queue == test_case['expected_processing'], \
                    f"处理队列名称不匹配: {queue.processing_queue} != {test_case['expected_processing']}"
                assert queue.failed_queue == test_case['expected_failed'], \
                    f"失败队列名称不匹配: {queue.failed_queue} != {test_case['expected_failed']}"
                
                print("  测试通过")
            except Exception as e:
                print(f"  测试失败: {e}")
                traceback.print_exc()
                return False
            
            print()
        
        print("Redis 队列命名修复测试通过！")
        return True
        
    except Exception as e:
        print(f"Redis 队列命名修复测试失败: {e}")
        traceback.print_exc()
        return False


def main():
    """主测试函数"""
    print("开始双重 crawlo 前缀问题修复测试...")
    print("=" * 50)
    
    try:
        # 测试 Redis 队列命名修复
        redis_test_success = test_redis_queue_naming()
        print()
        
        print("=" * 50)
        if redis_test_success:
            print("Redis 队列命名修复测试通过！双重 crawlo 前缀问题已修复")
        else:
            print("Redis 队列命名修复测试失败，请检查实现")
            return 1
            
    except Exception as e:
        print("=" * 50)
        print(f"测试过程中发生异常: {e}")
        traceback.print_exc()
        return 1
    
    return 0


if __name__ == "__main__":
    exit_code = main()
    sys.exit(exit_code)


#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
Crawlo 事件系统
==============
定义框架中的所有事件类型，支持类型安全和IDE自动补全。
"""
from enum import Enum


class CrawlerEvent(str, Enum):
    """
    爬虫事件枚举
    
    所有事件都应该使用此枚举类型，以获得：
    - 类型安全：避免拼写错误
    - IDE支持：自动补全和提示
    - 文档化：集中管理所有事件
    
    使用示例：
        >>> from crawlo.event import CrawlerEvent
        >>> await subscriber.notify(CrawlerEvent.SPIDER_OPENED, spider)
    """
    
    # 爬虫生命周期事件
    SPIDER_OPENED = "spider_opened"      # 爬虫启动
    SPIDER_CLOSED = "spider_closed"      # 爬虫关闭
    SPIDER_ERROR = "spider_error"        # 爬虫错误
    
    # 请求相关事件
    REQUEST_SCHEDULED = "request_scheduled"  # 请求已调度
    IGNORE_REQUEST = "ignore_request"        # 请求被忽略
    
    # 响应相关事件
    RESPONSE_RECEIVED = "response_received"  # 响应已接收
    
    # Item相关事件
    ITEM_SUCCESSFUL = "item_successful"  # Item处理成功
    ITEM_DISCARD = "item_discard"        # Item被丢弃


# 导出所有公共API
__all__ = [
    'CrawlerEvent',
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
调试处理队列清理问题
深入分析为什么处理队列没有被正确清理
"""
import asyncio
import sys
import os
import traceback
import time

# 添加项目根目录到路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.queue.redis_priority_queue import RedisPriorityQueue
from crawlo.network.request import Request


async def test_processing_queue_debug():
    """调试处理队列清理问题"""
    print("开始调试处理队列清理问题...")
    print("=" * 50)
    
    queue = None
    try:
        # 创建Redis队列实例
        queue = RedisPriorityQueue(
            redis_url="redis://127.0.0.1:6379/15",  # 使用测试数据库
            queue_name="test:processing:debug",
            module_name="test_processing_debug"
        )
        
        # 连接Redis
        await queue.connect()
        print("✅ Redis连接成功")
        
        # 确保Redis连接存在
        if not queue._redis:
            print("❌ Redis连接失败")
            return False
        
        # 清理可能存在的旧数据
        await queue._redis.delete(
            queue.queue_name,
            f"{queue.queue_name}:data",
            queue.processing_queue,
            f"{queue.processing_queue}:data"
        )
        print("✅ 旧数据清理完成")
        
        # 添加测试请求
        test_requests = [
            Request(url="https://example.com/test1"),
            Request(url="https://example.com/test2")
        ]
        
        print("\n--- 添加测试请求 ---")
        for i, request in enumerate(test_requests):
            success = await queue.put(request, priority=0)
            if success:
                print(f"✅ 请求{i+1}已添加到队列: {request.url}")
            else:
                print(f"❌ 请求{i+1}添加失败")
                return False
        
        # 验证主队列大小
        main_queue_size = await queue._redis.zcard(queue.queue_name)
        print(f"✅ 主队列大小: {main_queue_size}")
        
        # 从主队列获取任务（会自动移动到处理队列）
        print("\n--- 模拟任务处理 ---")
        processed_requests = []
        for i in range(len(test_requests)):
            request = await queue.get(timeout=1.0)
            if request:
                print(f"✅ 任务{i+1}已从主队列取出并移动到处理队列: {request.url}")
                processed_requests.append(request)
                
                # 查看处理队列中的具体数据
                processing_keys = await queue._redis.zrange(queue.processing_queue, 0, -1)
                print(f"   处理队列中的key: {processing_keys}")
            else:
                print(f"❌ 无法获取任务{i+1}")
                return False
        
        # 验证处理队列不为空
        if queue._redis:
            processing_queue_size = await queue._redis.zcard(queue.processing_queue)
            processing_data_size = await queue._redis.hlen(f"{queue.processing_queue}:data")
            print(f"✅ 处理队列大小: {processing_queue_size}")
            print(f"✅ 处理队列数据大小: {processing_data_size}")
            
            # 显示处理队列中的具体数据
            processing_keys = await queue._redis.zrange(queue.processing_queue, 0, -1)
            print(f"✅ 处理队列中的key: {processing_keys}")
            
            processing_data = await queue._redis.hgetall(f"{queue.processing_queue}:data")
            print(f"✅ 处理队列数据keys: {list(processing_data.keys())}")
        
        # 现在调用ack方法确认任务完成，但逐个处理并查看详细信息
        print("\n--- 逐个调用ack方法确认任务完成 ---")
        for i, request in enumerate(processed_requests):
            print(f"处理请求 {i+1}: {request.url}")
            
            # 查看请求的key
            request_key = queue._get_request_key(request)
            print(f"  请求key: {request_key}")
            
            # 调用ack之前查看处理队列
            before_keys = await queue._redis.zrange(queue.processing_queue, 0, -1)
            print(f"  ack之前处理队列keys: {before_keys}")
            
            # 调用ack方法
            await queue.ack(request)
            print(f"  ✅ 请求{i+1}已确认完成: {request.url}")
            
            # 调用ack之后查看处理队列
            after_keys = await queue._redis.zrange(queue.processing_queue, 0, -1)
            print(f"  ack之后处理队列keys: {after_keys}")
        
        # 验证处理队列是否为空
        final_processing_queue_size = await queue._redis.zcard(queue.processing_queue)
        final_processing_data_size = await queue._redis.hlen(f"{queue.processing_queue}:data")
        print(f"\n✅ 最终处理队列大小: {final_processing_queue_size}")
        print(f"✅ 最终处理队列数据大小: {final_processing_data_size}")
        
        if final_processing_queue_size == 0 and final_processing_data_size == 0:
            print("✅ 处理队列已正确清理")
        else:
            print(f"❌ 处理队列清理失败，队列大小: {final_processing_queue_size}, 数据大小: {final_processing_data_size}")
            
            # 显示剩余的key
            remaining_keys = await queue._redis.zrange(queue.processing_queue, 0, -1)
            print(f"  剩余的key: {remaining_keys}")
            
            remaining_data = await queue._redis.hgetall(f"{queue.processing_queue}:data")
            print(f"  剩余的数据keys: {list(remaining_data.keys())}")
            
            return False
        
        print("\n✅ 处理队列调试测试通过！")
        return True
        
    except Exception as e:
        print(f"❌ 测试失败: {e}")
        traceback.print_exc()
        return False
    finally:
        # 清理测试数据
        if queue and queue._redis:
            await queue._redis.delete(
                queue.queue_name,
                f"{queue.queue_name}:data",
                queue.processing_queue,
                f"{queue.processing_queue}:data"
            )


if __name__ == "__main__":
    asyncio.run(test_processing_queue_debug())


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
分布式采集测试脚本
用于验证分布式采集功能是否正常工作
"""

import asyncio
import sys
import os
import time

# 添加项目根目录到 Python 路径
project_root = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, project_root)

# 切换到项目根目录
os.chdir(project_root)

from crawlo.crawler import CrawlerProcess
from crawlo.utils.log import get_logger

logger = get_logger(__name__)


async def test_distributed_crawling():
    """测试分布式采集功能"""
    logger.info("开始测试分布式采集功能...")

    # 创建爬虫进程并应用配置
    try:
        # 确保 spider 模块被正确导入
        spider_modules = ['ofweek_distributed.spiders']
        process = CrawlerProcess(spider_modules=spider_modules)
        logger.info("爬虫进程初始化成功")

        # 运行指定的爬虫，使用正确的爬虫名称
        result = await process.crawl('of_week_distributed')
        logger.info(f"爬虫运行完成，结果: {result}")

    except Exception as e:
        logger.error(f"运行失败: {e}")
        import traceback
        traceback.print_exc()
        sys.exit(1)


def main():
    """主函数"""
    start_time = time.time()
    logger.info("开始分布式采集测试")

    try:
        asyncio.run(test_distributed_crawling())
        end_time = time.time()
        logger.info(f"分布式采集测试完成，耗时: {end_time - start_time:.2f} 秒")
    except KeyboardInterrupt:
        logger.info("用户中断测试")
    except Exception as e:
        logger.error(f"测试过程中发生错误: {e}")
        sys.exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
分布式场景下的多节点去重测试
==========================

测试目标：
1. 验证多个节点同时运行时，Redis 去重机制是否正常工作
2. 确认相同 URL 不会被多个节点重复处理
3. 验证 AioRedisFilter 在分布式场景下的一致性
4. 检查数据项去重管道的有效性

测试方法：
- 启动多个爬虫实例（模拟多节点）
- 使用相同的 Redis 配置
- 爬取相同的 URL 列表
- 统计实际处理的 URL 数量
- 验证是否存在重复处理

使用方式：
    python tests/distributed_dedup_test.py
"""

import asyncio
import sys
import os
import time
import redis
from pathlib import Path
from typing import List, Dict, Set
from collections import defaultdict

# 添加项目根目录到 Python 路径
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))

from crawlo import Spider
from crawlo.network.request import Request
from crawlo.crawler import Crawler
from crawlo.settings.setting_manager import SettingManager as Settings


class DedupTestSpider(Spider):
    """专门用于测试去重的爬虫"""
    
    name = 'dedup_test_spider'
    
    # 测试 URL 列表（包含重复）
    test_urls = [
        'http://httpbin.org/delay/1',
        'http://httpbin.org/delay/2',
        'http://httpbin.org/delay/1',  # 重复
        'http://httpbin.org/html',
        'http://httpbin.org/json',
        'http://httpbin.org/html',  # 重复
        'http://httpbin.org/uuid',
        'http://httpbin.org/delay/1',  # 重复
        'http://httpbin.org/json',  # 重复
        'http://httpbin.org/uuid',  # 重复
    ]
    
    def __init__(self, instance_id: int = 0, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instance_id = instance_id
        self.processed_urls: Set[str] = set()
        self.duplicate_count = 0
    
    def start_requests(self):
        """生成初始请求"""
        self.logger.info(f"[实例 {self.instance_id}] 开始生成请求")
        for url in self.test_urls:
            yield Request(
                url=url,
                callback=self.parse,
                meta={'instance_id': self.instance_id}
            )
    
    async def parse(self, response):
        """解析响应"""
        url = response.url
        instance_id = response.meta.get('instance_id', self.instance_id)
        
        # 记录处理的 URL
        if url in self.processed_urls:
            self.duplicate_count += 1
            self.logger.warning(
                f"[实例 {instance_id}] ⚠️ 检测到重复处理: {url}"
            )
        else:
            self.processed_urls.add(url)
            self.logger.info(
                f"[实例 {instance_id}] ✓ 处理新URL: {url}"
            )
        
        # 返回数据项
        yield {
            'url': url,
            'instance_id': instance_id,
            'timestamp': time.time(),
            'status': response.status_code,  # 使用 status_code
        }


class DistributedDedupTest:
    """分布式去重测试管理器"""
    
    def __init__(self, num_instances: int = 3):
        """
        初始化测试
        
        :param num_instances: 模拟的节点数量
        """
        self.num_instances = num_instances
        self.redis_config = {
            'host': '127.0.0.1',
            'port': 6379,
            'db': 15,  # 使用独立的数据库避免污染
            'password': '',
        }
        self.project_name = 'dedup_test'
        self.results: Dict[int, Dict] = {}
        
    def _check_redis_connection(self) -> bool:
        """检查 Redis 连接"""
        try:
            r = redis.Redis(
                host=self.redis_config['host'],
                port=self.redis_config['port'],
                db=self.redis_config['db'],
                password=self.redis_config['password'] or None,
                decode_responses=True,
                socket_connect_timeout=5,
            )
            r.ping()
            print(f"✓ Redis 连接正常: {self.redis_config['host']}:{self.redis_config['port']}/{self.redis_config['db']}")
            return True
        except Exception as e:
            print(f"✗ Redis 连接失败: {e}")
            print(f"  请确保 Redis 服务正在运行")
            return False
    
    def _cleanup_redis(self):
        """清理 Redis 中的测试数据"""
        try:
            r = redis.Redis(
                host=self.redis_config['host'],
                port=self.redis_config['port'],
                db=self.redis_config['db'],
                password=self.redis_config['password'] or None,
                decode_responses=True,
            )
            
            # 清理所有与项目相关的 key
            pattern = f"crawlo:{self.project_name}:*"
            keys = list(r.scan_iter(pattern))
            if keys:
                deleted = r.delete(*keys)
                print(f"✓ 清理了 {deleted} 个 Redis 键")
            else:
                print(f"✓ 没有需要清理的 Redis 键")
                
        except Exception as e:
            print(f"⚠ Redis 清理失败: {e}")
    
    def _create_settings(self, instance_id: int) -> Settings:
        """创建爬虫配置"""
        settings = Settings()
        
        # 项目基本配置
        settings.set('PROJECT_NAME', self.project_name)
        settings.set('RUN_MODE', 'distributed')
        
        # Redis 配置
        settings.set('REDIS_HOST', self.redis_config['host'])
        settings.set('REDIS_PORT', self.redis_config['port'])
        settings.set('REDIS_DB', self.redis_config['db'])
        settings.set('REDIS_PASSWORD', self.redis_config['password'])
        
        # 构建 Redis URL
        if self.redis_config['password']:
            redis_url = (
                f"redis://:{self.redis_config['password']}@"
                f"{self.redis_config['host']}:{self.redis_config['port']}/{self.redis_config['db']}"
            )
        else:
            redis_url = (
                f"redis://{self.redis_config['host']}:{self.redis_config['port']}/{self.redis_config['db']}"
            )
        settings.set('REDIS_URL', redis_url)
        
        # 队列和过滤器配置
        settings.set('QUEUE_TYPE', 'redis')
        settings.set('FILTER_CLASS', 'crawlo.filters.aioredis_filter.AioRedisFilter')
        settings.set('DEFAULT_DEDUP_PIPELINE', 'crawlo.pipelines.redis_dedup_pipeline.RedisDedupPipeline')
        
        # 管道配置
        settings.set('PIPELINES', [
            'crawlo.pipelines.redis_dedup_pipeline.RedisDedupPipeline',
        ])
        
        # 并发配置
        settings.set('CONCURRENCY', 5)
        settings.set('DOWNLOAD_DELAY', 0.5)
        
        # 日志配置
        settings.set('LOG_LEVEL', 'INFO')
        settings.set('LOG_FILE', f'logs/dedup_test_instance_{instance_id}.log')
        settings.set('LOG_ENCODING', 'utf-8')
        settings.set('STATS_DUMP', True)
        
        # 禁用某些扩展以简化测试
        settings.set('EXTENSIONS', [])
        
        return settings
    
    async def _run_instance(self, instance_id: int):
        """运行单个爬虫实例"""
        print(f"\n{'='*60}")
        print(f"启动实例 {instance_id}")
        print(f"{'='*60}")
        
        # 创建配置
        settings = self._create_settings(instance_id)
        
        # 创建爬虫实例
        spider = DedupTestSpider(instance_id=instance_id)
        
        # 创建 Crawler（传入爬虫类而不是实例）
        crawler = Crawler(DedupTestSpider, settings)
        # 手动设置spider实例的instance_id
        crawler._spider = spider
        
        try:
            # 运行爬虫
            await crawler.crawl()
            
            # 收集统计信息
            stats = crawler.stats.get_stats() if crawler.stats else {}
            
            self.results[instance_id] = {
                'spider': spider,
                'stats': stats,
                'processed_urls': spider.processed_urls.copy(),
                'duplicate_count': spider.duplicate_count,
            }
            
            print(f"\n[实例 {instance_id}] 完成:")
            print(f"  - 处理的 URL 数量: {len(spider.processed_urls)}")
            print(f"  - 检测到的重复: {spider.duplicate_count}")
            
        except Exception as e:
            print(f"\n[实例 {instance_id}] 运行失败: {e}")
            import traceback
            traceback.print_exc()
        finally:
            # 清理资源
            await crawler.close()
    
    async def run_parallel_test(self):
        """并行运行多个实例（真实的分布式场景）"""
        print(f"\n{'='*60}")
        print(f"并行测试：同时启动 {self.num_instances} 个实例")
        print(f"{'='*60}")
        
        # 创建所有任务
        tasks = [
            self._run_instance(i)
            for i in range(self.num_instances)
        ]
        
        # 并行执行
        await asyncio.gather(*tasks)
    
    async def run_sequential_test(self):
        """顺序运行多个实例（验证基本去重）"""
        print(f"\n{'='*60}")
        print(f"顺序测试：依次运行 {self.num_instances} 个实例")
        print(f"{'='*60}")
        
        for i in range(self.num_instances):
            await self._run_instance(i)
            # 等待一小段时间
            await asyncio.sleep(1)
    
    def _analyze_results(self):
        """分析测试结果"""
        print(f"\n{'='*60}")
        print(f"测试结果分析")
        print(f"{'='*60}")
        
        if not self.results:
            print("⚠ 没有收集到任何结果")
            return
        
        # 统计所有实例处理的 URL
        all_processed_urls: Set[str] = set()
        total_duplicates = 0
        total_requests = 0
        
        print(f"\n各实例统计:")
        for instance_id, result in sorted(self.results.items()):
            spider = result['spider']
            stats = result['stats']
            
            processed_count = len(result['processed_urls'])
            duplicate_count = result['duplicate_count']
            
            all_processed_urls.update(result['processed_urls'])
            total_duplicates += duplicate_count
            total_requests += stats.get('request/success_count', 0)
            
            print(f"\n  实例 {instance_id}:")
            print(f"    - 处理的唯一 URL: {processed_count}")
            print(f"    - 本地检测到的重复: {duplicate_count}")
            print(f"    - 成功的请求: {stats.get('request/success_count', 0)}")
            print(f"    - 失败的请求: {stats.get('request/failed_count', 0)}")
        
        # 检查 Redis 中的数据
        print(f"\n{'='*60}")
        print(f"Redis 数据检查:")
        print(f"{'='*60}")
        
        try:
            r = redis.Redis(
                host=self.redis_config['host'],
                port=self.redis_config['port'],
                db=self.redis_config['db'],
                password=self.redis_config['password'] or None,
                decode_responses=True,
            )
            
            # 检查过滤器指纹
            filter_key = f"crawlo:{self.project_name}:filter:fingerprint"
            filter_count = r.scard(filter_key)
            print(f"  - 过滤器指纹数量: {filter_count}")
            
            # 检查数据项指纹
            item_key = f"crawlo:{self.project_name}:item:fingerprint"
            item_count = r.scard(item_key)
            print(f"  - 数据项指纹数量: {item_count}")
            
            # 检查队列
            queue_key = f"crawlo:{self.project_name}:queue:requests"
            queue_len = r.zcard(queue_key)
            print(f"  - 剩余队列长度: {queue_len}")
            
        except Exception as e:
            print(f"  ⚠ Redis 检查失败: {e}")
        
        # 总体统计
        print(f"\n{'='*60}")
        print(f"总体统计:")
        print(f"{'='*60}")
        
        unique_urls_count = len(all_processed_urls)
        expected_unique_urls = len(set(DedupTestSpider.test_urls))
        
        print(f"  - 实例数量: {len(self.results)}")
        print(f"  - 所有实例处理的唯一 URL: {unique_urls_count}")
        print(f"  - 预期的唯一 URL 数量: {expected_unique_urls}")
        print(f"  - 总请求数: {total_requests}")
        print(f"  - 本地检测到的总重复: {total_duplicates}")
        
        # 验证去重效果
        print(f"\n{'='*60}")
        print(f"去重效果验证:")
        print(f"{'='*60}")
        
        # 关键验证：所有实例处理的 URL 总数应该等于唯一 URL 数
        if unique_urls_count == expected_unique_urls:
            print(f"  ✓ 测试通过！")
            print(f"    所有实例共处理了 {unique_urls_count} 个唯一 URL")
            print(f"    没有任何 URL 被多个节点重复处理")
            return True
        else:
            print(f"  ✗ 测试失败！")
            print(f"    预期处理 {expected_unique_urls} 个唯一 URL")
            print(f"    实际处理 {unique_urls_count} 个唯一 URL")
            
            # 检查是否有 URL 被遗漏或重复
            expected_urls = set(DedupTestSpider.test_urls)
            missing_urls = expected_urls - all_processed_urls
            
            if missing_urls:
                print(f"    遗漏的 URL: {missing_urls}")
            
            if unique_urls_count > expected_unique_urls:
                print(f"    可能存在重复处理")
            
            return False
    
    async def run(self, mode: str = 'parallel'):
        """
        运行测试
        
        :param mode: 测试模式 'parallel' 或 'sequential'
        """
        print(f"\n{'='*70}")
        print(f"分布式去重测试")
        print(f"{'='*70}")
        print(f"测试模式: {mode}")
        print(f"实例数量: {self.num_instances}")
        print(f"Redis: {self.redis_config['host']}:{self.redis_config['port']}/{self.redis_config['db']}")
        
        # 检查 Redis 连接
        if not self._check_redis_connection():
            print("\n⚠ Redis 不可用，测试终止")
            return False
        
        # 清理旧数据
        print(f"\n清理 Redis 旧数据...")
        self._cleanup_redis()
        
        # 运行测试
        try:
            if mode == 'parallel':
                await self.run_parallel_test()
            else:
                await self.run_sequential_test()
            
            # 等待一小段时间确保所有数据都写入 Redis
            await asyncio.sleep(2)
            
            # 分析结果
            return self._analyze_results()
            
        except Exception as e:
            print(f"\n✗ 测试执行失败: {e}")
            import traceback
            traceback.print_exc()
            return False
        finally:
            # 可选：测试后清理数据
            print(f"\n清理测试数据...")
            self._cleanup_redis()


async def main():
    """主函数"""
    import argparse
    
    parser = argparse.ArgumentParser(description='分布式去重测试')
    parser.add_argument(
        '--instances',
        type=int,
        default=3,
        help='模拟的节点数量（默认: 3）'
    )
    parser.add_argument(
        '--mode',
        choices=['parallel', 'sequential'],
        default='parallel',
        help='测试模式: parallel（并行）或 sequential（顺序）'
    )
    
    args = parser.parse_args()
    
    # 创建并运行测试
    test = DistributedDedupTest(num_instances=args.instances)
    success = await test.run(mode=args.mode)
    
    # 返回退出码
    sys.exit(0 if success else 1)


if __name__ == '__main__':
    asyncio.run(main())



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
# @Time    : 2025-09-10 22:00
# @Author  : crawl-coder
# @Desc    : Crawlo框架工具包集合
"""

# 日期工具封装
from .date_tools import (
    TimeUtils,
    parse_time,
    format_time,
    time_diff,
    to_timestamp,
    to_datetime,
    now,
    to_timezone,
    to_utc,
    to_local,
    from_timestamp_with_tz
)

# 数据清洗工具封装
from .text_cleaner import (
    TextCleaner,
    remove_html_tags,
    decode_html_entities,
    remove_extra_whitespace,
    remove_special_chars,
    normalize_unicode,
    clean_text,
    extract_numbers,
    extract_emails,
    extract_urls
)

# 分布式协调工具
from .distributed_coordinator import (
    TaskDistributor,
    DeduplicationTool,
    DistributedCoordinator,
    generate_task_id,
    claim_task,
    report_task_status,
    get_cluster_info,
    generate_pagination_tasks,
    distribute_tasks
)

__all__ = [
    # 日期工具
    "TimeUtils",
    "parse_time",
    "format_time",
    "time_diff",
    "to_timestamp",
    "to_datetime",
    "now",
    "to_timezone",
    "to_utc",
    "to_local",
    "from_timestamp_with_tz",
    
    # 数据清洗工具
    "TextCleaner",
    "remove_html_tags",
    "decode_html_entities",
    "remove_extra_whitespace",
    "remove_special_chars",
    "normalize_unicode",
    "clean_text",
    "extract_numbers",
    "extract_emails",
    "extract_urls",
    
    # 分布式协调工具
    "TaskDistributor",
    "DeduplicationTool",
    "DistributedCoordinator",
    "generate_task_id",
    "claim_task",
    "report_task_status",
    "get_cluster_info",
    "generate_pagination_tasks",
    "distribute_tasks"
]


#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Crawlo框架高级工具使用示例
"""
from crawlo.tools import (
    # 数据处理工具
    clean_text,
    format_currency,
    validate_email,
    validate_url,
    check_data_integrity,
    
    # 重试机制
    RetryMechanism,
    retry,
    exponential_backoff,
    
    # 反爬虫应对工具
    AntiCrawler,
    rotate_proxy,
    handle_captcha,
    detect_rate_limiting,
    
    # 分布式协调工具
    generate_pagination_tasks,
    distribute_tasks,
    DistributedCoordinator
)


def demo_data_processing_tools():
    """演示数据处理工具的使用"""
    print("=== 数据处理工具演示 ===\n")
    
    # 数据清洗
    dirty_text = "<p>这是一个&nbsp;<b>测试</b>&amp;文本</p>"
    clean_result = clean_text(dirty_text)
    print(f"清洗文本: {dirty_text} -> {clean_result}")
    
    # 数据格式化
    price = 1234.567
    formatted_price = format_currency(price, "¥", 2)
    print(f"格式化货币: {price} -> {formatted_price}")
    
    # 字段验证
    email = "test@example.com"
    is_valid_email = validate_email(email)
    print(f"验证邮箱: {email} -> {'有效' if is_valid_email else '无效'}")
    
    url = "https://example.com"
    is_valid_url = validate_url(url)
    print(f"验证URL: {url} -> {'有效' if is_valid_url else '无效'}")
    
    # 数据完整性检查
    data = {
        "name": "张三",
        "email": "test@example.com",
        "phone": "13812345678"
    }
    required_fields = ["name", "email", "phone", "address"]
    integrity_result = check_data_integrity(data, required_fields)
    print(f"数据完整性检查: {integrity_result}")
    
    print()


def demo_retry_mechanism():
    """演示重试机制的使用"""
    print("=== 重试机制演示 ===\n")
    
    # 指数退避
    for attempt in range(5):
        delay = exponential_backoff(attempt)
        print(f"重试次数 {attempt}: 延迟 {delay:.2f} 秒")
    
    # 重试装饰器示例
    @retry(max_retries=3)
    def unreliable_function():
        import random
        if random.random() < 0.7:  # 70%概率失败
            raise ConnectionError("网络连接失败")
        return "成功执行"
    
    try:
        result = unreliable_function()
        print(f"函数执行结果: {result}")
    except Exception as e:
        print(f"函数执行失败: {e}")
    
    print()


def demo_anti_crawler_tools():
    """演示反爬虫应对工具的使用"""
    print("=== 反爬虫应对工具演示 ===\n")
    
    # 反爬虫工具
    anti_crawler = AntiCrawler()
    
    # 获取随机User-Agent
    user_agent = anti_crawler.get_random_user_agent()
    print(f"随机User-Agent: {user_agent[:50]}...")
    
    # 轮换代理
    proxy = anti_crawler.rotate_proxy()
    print(f"轮换代理: {proxy}")
    
    # 检测验证码
    response_with_captcha = "请输入验证码进行验证"
    has_captcha = anti_crawler.handle_captcha(response_with_captcha)
    print(f"检测验证码: {response_with_captcha} -> {'需要验证码' if has_captcha else '无需验证码'}")
    
    # 检测频率限制
    status_code = 429  # Too Many Requests
    response_headers = {"Retry-After": "60"}
    is_rate_limited = anti_crawler.detect_rate_limiting(status_code, response_headers)
    print(f"检测频率限制: 状态码{status_code} -> {'被限制' if is_rate_limited else '未限制'}")
    
    print()


def demo_distributed_coordinator_tools():
    """演示分布式协调工具的使用"""
    print("=== 分布式协调工具演示 ===\n")
    
    # 生成分页任务
    base_url = "https://example.com/products"
    pagination_tasks = generate_pagination_tasks(base_url, 1, 5)
    print(f"生成分页任务 ({len(pagination_tasks)} 个):")
    for i, task in enumerate(pagination_tasks[:3]):  # 只显示前3个
        print(f"  {i+1}. {task}")
    if len(pagination_tasks) > 3:
        print(f"  ... 还有 {len(pagination_tasks) - 3} 个任务")
    
    # 任务分发
    tasks = list(range(1, 21))  # 20个任务
    distributed = distribute_tasks(tasks, 4)  # 分发给4个工作节点
    print(f"\n任务分发 (20个任务分发给4个工作节点):")
    for i, worker_tasks in enumerate(distributed):
        print(f"  工作节点 {i+1}: {len(worker_tasks)} 个任务 -> {worker_tasks}")
    
    # 分布式协调器
    coordinator = DistributedCoordinator()
    cluster_info = coordinator.get_cluster_info()
    print(f"\n集群信息: {cluster_info}")
    
    print()


def demo_in_spider():
    """演示在爬虫中使用高级工具"""
    print("=== 在爬虫中使用高级工具 ===\n")
    print("在爬虫项目中，您可以这样使用高级工具:")
    print("""
import asyncio
from crawlo import Spider, Request
from crawlo.tools import (
    clean_text,
    validate_email,
    AntiCrawler,
    DistributedCoordinator,
    retry
)

class AdvancedSpider(Spider):
    def __init__(self):
        super().__init__()
        self.anti_crawler = AntiCrawler()
        self.coordinator = DistributedCoordinator()
    
    def start_requests(self):
        # 生成分页任务
        base_url = "https://api.example.com/products"
        pagination_tasks = self.coordinator.generate_pagination_tasks(base_url, 1, 100)
        
        for url in pagination_tasks:
            # 直接使用带认证的代理URL（框架原生支持）
            request = Request(
                url, 
                callback=self.parse,
                proxy="http://user:pass@proxy.example.com:8080"  # 所有下载器都支持
            )
            yield request
    
    @retry(max_retries=3)
    async def parse(self, response):
        # 检查是否遇到验证码
        if self.anti_crawler.handle_captcha(response.text):
            # 处理验证码逻辑
            print("遇到验证码，需要处理")
            return
            
        # 提取数据
        products = response.css('.product-item')
        for product in products:
            name = product.css('.product-name::text').get()
            email = product.css('.contact-email::text').get()
            
            # 数据清洗和验证
            clean_name = clean_text(name) if name else None
            is_valid_email = validate_email(email) if email else False
            
            # 检查数据是否重复
            if not await self.coordinator.is_duplicate({"name": clean_name}):
                await self.coordinator.add_to_dedup({"name": clean_name})
                # 处理产品数据...
                pass
    """)


if __name__ == '__main__':
    # 运行演示
    demo_data_processing_tools()
    demo_retry_mechanism()
    demo_anti_crawler_tools()
    demo_distributed_coordinator_tools()
    demo_in_spider()


#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
性能监控工具
提供系统性能监控和资源使用情况跟踪
"""
import asyncio
import time
from functools import wraps
from typing import Dict, Any

import psutil

from crawlo.utils.error_handler import ErrorHandler
from crawlo.logging import get_logger


class PerformanceMonitor:
    """性能监控器"""
    
    def __init__(self, logger_name: str = __name__):
        self.logger = get_logger(logger_name)
        self.error_handler = ErrorHandler(logger_name)
        self.process = psutil.Process()
        self.start_time = time.time()
        
        # 性能指标
        self.metrics = {
            'cpu_usage': [],
            'memory_usage': [],
            'network_io': [],
            'disk_io': []
        }
    
    def get_system_metrics(self) -> Dict[str, Any]:
        """
        获取系统性能指标
        
        Returns:
            包含各种性能指标的字典
        """
        try:
            # CPU使用率
            cpu_percent = psutil.cpu_percent(interval=1)
            
            # 内存使用情况
            memory = psutil.virtual_memory()
            
            # 网络IO
            net_io = psutil.net_io_counters()
            
            # 磁盘IO
            disk_io = psutil.disk_io_counters()
            
            # 进程特定信息
            process_memory = self.process.memory_info()
            process_cpu = self.process.cpu_percent()
            
            return {
                'timestamp': time.time(),
                'uptime': time.time() - self.start_time,
                'cpu': {
                    'percent': cpu_percent,
                    'count': psutil.cpu_count(),
                    'freq': psutil.cpu_freq()._asdict() if psutil.cpu_freq() else {}
                },
                'memory': {
                    'total': memory.total,
                    'available': memory.available,
                    'percent': memory.percent,
                    'used': memory.used,
                    'free': memory.free
                },
                'process': {
                    'memory_rss': process_memory.rss,
                    'memory_vms': process_memory.vms,
                    'cpu_percent': process_cpu,
                    'num_threads': self.process.num_threads(),
                    'num_fds': self.process.num_fds() if hasattr(self.process, 'num_fds') else 0
                },
                'network': {
                    'bytes_sent': net_io.bytes_sent,
                    'bytes_recv': net_io.bytes_recv,
                    'packets_sent': net_io.packets_sent,
                    'packets_recv': net_io.packets_recv
                },
                'disk': {
                    'read_bytes': disk_io.read_bytes,
                    'write_bytes': disk_io.write_bytes,
                    'read_count': disk_io.read_count,
                    'write_count': disk_io.write_count
                }
            }
        except Exception as e:
            self.error_handler.handle_error(
                e, 
                context="获取系统性能指标失败", 
                raise_error=False
            )
            return {}
    
    def log_system_metrics(self, detailed: bool = False):
        """
        记录系统性能指标
        
        Args:
            detailed: 是否记录详细信息
        """
        try:
            metrics = self.get_system_metrics()
            if not metrics:
                return
            
            # 基本信息
            basic_info = (
                f"系统性能指标 | "
                f"CPU: {metrics['cpu']['percent']:.1f}% | "
                f"内存: {metrics['memory']['percent']:.1f}% | "
                f"进程CPU: {metrics['process']['cpu_percent']:.1f}% | "
                f"进程内存: {metrics['process']['memory_rss'] / 1024 / 1024:.1f}MB"
            )
            self.logger.info(basic_info)
            
            # 详细信息
            if detailed:
                detailed_info = (
                    f"   详细信息:\n"
                    f"   - CPU: {metrics['cpu']['count']} 核心\n"
                    f"   - 内存: 总计 {metrics['memory']['total'] / 1024 / 1024 / 1024:.1f}GB, "
                    f"可用 {metrics['memory']['available'] / 1024 / 1024 / 1024:.1f}GB\n"
                    f"   - 网络: 发送 {metrics['network']['bytes_sent'] / 1024 / 1024:.1f}MB, "
                    f"接收 {metrics['network']['bytes_recv'] / 1024 / 1024:.1f}MB\n"
                    f"   - 磁盘: 读取 {metrics['disk']['read_bytes'] / 1024 / 1024:.1f}MB, "
                    f"写入 {metrics['disk']['write_bytes'] / 1024 / 1024:.1f}MB"
                )
                self.logger.debug(detailed_info)
        except Exception as e:
            self.error_handler.handle_error(
                e, 
                context="记录系统性能指标失败", 
                raise_error=False
            )
    
    def start_monitoring(self, interval: int = 60, detailed: bool = False):
        """
        开始定期监控
        
        Args:
            interval: 监控间隔（秒）
            detailed: 是否记录详细信息
        """
        async def monitor_loop():
            while True:
                try:
                    self.log_system_metrics(detailed)
                    await asyncio.sleep(interval)
                except asyncio.CancelledError:
                    break
                except Exception as e:
                    self.logger.error(f"监控循环错误: {e}")
        
        # 启动监控任务
        self.monitor_task = asyncio.create_task(monitor_loop())
        self.logger.info(f"开始性能监控，间隔: {interval}秒")
    
    async def stop_monitoring(self):
        """停止监控"""
        if hasattr(self, 'monitor_task') and self.monitor_task:
            self.monitor_task.cancel()
            try:
                await self.monitor_task
            except asyncio.CancelledError:
                pass
            self.logger.info("性能监控已停止")


class PerformanceTimer:
    """性能计时器"""
    
    def __init__(self, name: str = "timer"):
        self.name = name
        self.start_time = None
        self.end_time = None
        self.logger = get_logger(f"{__name__}.{self.__class__.__name__}")
        self.error_handler = ErrorHandler(f"{__name__}.{self.__class__.__name__}")
    
    def start(self):
        """开始计时"""
        self.start_time = time.time()
        self.logger.debug(f"开始计时: {self.name}")
    
    def stop(self) -> float:
        """
        停止计时并返回耗时
        
        Returns:
            耗时（秒）
        """
        self.end_time = time.time()
        if self.start_time is None:
            raise RuntimeError("计时器未启动")
        
        elapsed = self.end_time - self.start_time
        self.logger.debug(f"停止计时: {self.name}, 耗时: {elapsed:.3f}秒")
        return elapsed
    
    def __enter__(self):
        self.start()
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            elapsed = self.stop()
            if exc_type is None:
                self.logger.info(f"{self.name} 执行成功，耗时: {elapsed:.3f}秒")
            else:
                self.logger.error(f"{self.name} 执行失败，耗时: {elapsed:.3f}秒")
        except Exception as e:
            self.error_handler.handle_error(
                e, 
                context=f"计时器退出时发生错误: {self.name}", 
                raise_error=False
            )


def performance_monitor_decorator(name: str = None, log_level: str = "INFO"):
    """
    装饰器：监控函数性能
    
    Args:
        name: 函数名称（如果为None则使用函数名）
        log_level: 日志级别
    """
    def decorator(func):
        @wraps(func)
        async def async_wrapper(*args, **kwargs):
            timer_name = name or f"{func.__module__}.{func.__name__}"
            logger = get_logger(timer_name)
            
            with PerformanceTimer(timer_name) as timer:
                if asyncio.iscoroutinefunction(func):
                    return await func(*args, **kwargs)
                else:
                    return func(*args, **kwargs)
        
        @wraps(func)
        def sync_wrapper(*args, **kwargs):
            timer_name = name or f"{func.__module__}.{func.__name__}"
            logger = get_logger(timer_name)
            
            with PerformanceTimer(timer_name) as timer:
                return func(*args, **kwargs)
        
        # 根据函数是否为异步函数返回相应的包装器
        import inspect
        if inspect.iscoroutinefunction(func):
            return async_wrapper
        else:
            return sync_wrapper
    
    return decorator


# 全局性能监控器实例
default_performance_monitor = PerformanceMonitor()


def monitor_performance(interval: int = 60, detailed: bool = False):
    """
    便捷函数：开始性能监控
    
    Args:
        interval: 监控间隔（秒）
        detailed: 是否记录详细信息
    """
    default_performance_monitor.start_monitoring(interval, detailed)


def get_current_metrics() -> Dict[str, Any]:
    """
    便捷函数：获取当前性能指标
    
    Returns:
        性能指标字典
    """
    return default_performance_monitor.get_system_metrics()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
测试Redis key中spider_name的使用
"""
import asyncio
import sys
import os

# 添加项目根目录到路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.utils.redis_manager import RedisKeyManager


def test_spider_name_in_keys():
    """测试spider_name是否正确地被使用在Redis key中"""
    print("测试Redis key中spider_name的使用...")
    print("=" * 50)
    
    # 测试1: 只有project_name
    print("1. 测试只有project_name的情况...")
    key_manager1 = RedisKeyManager("my_project")
    print(f"   请求队列key: {key_manager1.get_requests_queue_key()}")
    print(f"   处理队列key: {key_manager1.get_processing_queue_key()}")
    print(f"   过滤器key: {key_manager1.get_filter_fingerprint_key()}")
    print(f"   数据项key: {key_manager1.get_item_fingerprint_key()}")
    
    expected1 = "crawlo:my_project:queue:requests"
    actual1 = key_manager1.get_requests_queue_key()
    assert actual1 == expected1, f"期望: {expected1}, 实际: {actual1}"
    print("   ✅ 只有project_name的测试通过")
    
    # 测试2: 有project_name和spider_name
    print("\n2. 测试有project_name和spider_name的情况...")
    key_manager2 = RedisKeyManager("my_project", "my_spider")
    print(f"   请求队列key: {key_manager2.get_requests_queue_key()}")
    print(f"   处理队列key: {key_manager2.get_processing_queue_key()}")
    print(f"   过滤器key: {key_manager2.get_filter_fingerprint_key()}")
    print(f"   数据项key: {key_manager2.get_item_fingerprint_key()}")
    
    expected2 = "crawlo:my_project:my_spider:queue:requests"
    actual2 = key_manager2.get_requests_queue_key()
    assert actual2 == expected2, f"期望: {expected2}, 实际: {actual2}"
    print("   ✅ 有project_name和spider_name的测试通过")
    
    # 测试3: 动态设置spider_name
    print("\n3. 测试动态设置spider_name...")
    key_manager3 = RedisKeyManager("my_project")
    print(f"   设置spider_name前的请求队列key: {key_manager3.get_requests_queue_key()}")
    
    key_manager3.set_spider_name("dynamic_spider")
    print(f"   设置spider_name后的请求队列key: {key_manager3.get_requests_queue_key()}")
    
    expected3 = "crawlo:my_project:dynamic_spider:queue:requests"
    actual3 = key_manager3.get_requests_queue_key()
    assert actual3 == expected3, f"期望: {expected3}, 实际: {actual3}"
    print("   ✅ 动态设置spider_name的测试通过")
    
    # 测试4: 从配置创建key manager
    print("\n4. 测试从配置创建key manager...")
    class MockSettings:
        def get(self, key, default=None):
            settings_map = {
                'PROJECT_NAME': 'config_project',
                'SPIDER_NAME': 'config_spider'
            }
            return settings_map.get(key, default)
    
    settings = MockSettings()
    key_manager4 = RedisKeyManager.from_settings(settings)
    print(f"   从配置创建的请求队列key: {key_manager4.get_requests_queue_key()}")
    
    expected4 = "crawlo:config_project:config_spider:queue:requests"
    actual4 = key_manager4.get_requests_queue_key()
    assert actual4 == expected4, f"期望: {expected4}, 实际: {actual4}"
    print("   ✅ 从配置创建key manager的测试通过")
    
    print("\n✅ 所有测试通过！spider_name正确地被使用在Redis key中。")


if __name__ == "__main__":
    test_spider_name_in_keys()


#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
RequestIgnoreMiddleware 中间件
用于处理和记录被忽略的请求
"""
from crawlo.logging import get_logger
from crawlo.exceptions import IgnoreRequestError
from crawlo.event import CrawlerEvent


class RequestIgnoreMiddleware(object):
    """
    RequestIgnoreMiddleware 中间件
    用于处理和记录被忽略的请求，提供详细的统计信息
    """

    def __init__(self, stats):
        """
        初始化中间件
        
        Args:
            stats: 统计信息收集器
        """
        self.logger = get_logger(self.__class__.__name__)
        self.stats = stats

    @classmethod
    def create_instance(cls, crawler):
        """
        创建中间件实例
        
        Args:
            crawler: 爬虫实例
            
        Returns:
            RequestIgnoreMiddleware: 中间件实例
        """
        o = cls(stats=crawler.stats)
        crawler.subscriber.subscribe(o.request_ignore, event=CrawlerEvent.IGNORE_REQUEST)
        return o

    async def request_ignore(self, exc, request, _spider):
        """
        处理被忽略的请求事件
        
        Args:
            exc: 异常对象
            request: 被忽略的请求
            _spider: 爬虫实例
        """
        # 记录被忽略的请求
        self.logger.info(f'请求被忽略: {request.url}')
        self.stats.inc_value('request_ignore_count')
        
        # 记录忽略原因
        reason = getattr(exc, 'msg', 'unknown')
        if reason:
            self.stats.inc_value(f'request_ignore_count/reason/{reason}')
            
        # 记录请求的域名分布
        try:
            from urllib.parse import urlparse
            parsed_url = urlparse(request.url)
            domain = parsed_url.netloc
            if domain:
                self.stats.inc_value(f'request_ignore_count/domain/{domain}')
        except Exception:
            self.stats.inc_value('request_ignore_count/domain/invalid_url')

    @staticmethod
    def process_exception(_request, exc, _spider):
        """
        处理异常，识别IgnoreRequestError
        
        Args:
            _request: 请求对象
            exc: 异常对象
            _spider: 爬虫实例
            
        Returns:
            bool: 如果是IgnoreRequestError则返回True，否则返回None
        """
        if isinstance(exc, IgnoreRequestError):
            return True
        return None